#include <cmath>
#include <iostream>
#include "LibmFilter.h"

typedef LibmFilter::Vector Vector;
typedef LibmFilter::Matrix Matrix;

using namespace std;

#define NX	8
#define NY	4
#define NU	2

LibmFilter::LibmFilter() 
{
    // nx, nu, nw, nm, nv
    setDim(NX, NU, NX, NY, NY);
    Period = 0.001;		
    Mass = 78.2875;
    Gravity = 9.81;
    Height = 0.9;
	Initialized = 0;
}

void LibmFilter::makeProcess()
{
    Vector x_(NX);
    x_(1) = x(1) + x(2)*Period;
    x_(2) = x(2) + Gravity/Height*Period*x(1) - Gravity/Height*Period*x(3) + Period*x(4);
    x_(3) = x(3) + u(1)*Period;
    x_(4) = x(4);
    x_(5) = x(5) + x(6)*Period;
    x_(6) = x(6) + Gravity/Height*Period*x(5) - Gravity/Height*Period*x(7) + Period*x(8);
    x_(7) = x(7) + u(2)*Period;
    x_(8) = x(8);
    x.swap(x_);
}

void LibmFilter::makeMeasure()
{
    z(1)=x(1);
    z(2)=x(3);    
    z(3)=x(5);
    z(4)=x(7);
   
}

void LibmFilter::makeA(){
    // linear
}

void LibmFilter::makeH(){
    // linear
}

void LibmFilter::makeBaseA()
{
    int i,j;
    
    for(i=1;i<=NX;i++){
        for(j=1;j<=NX;j++) A(i,j) = 0.0;
        A(i,i) = 1.0;
    }

    A(1,2) = Period;

    A(2,1) = Gravity/Height*Period;
    A(2,3) = -Gravity/Height*Period;
    A(2,4) = 1/Mass*Period;

    A(5,6) = Period;

    A(6,5) = Gravity/Height*Period;
    A(6,7) = -Gravity/Height*Period;
    A(6,8) = 1/Mass*Period;
      
}

void LibmFilter::makeBaseW()
{
    int i,j;
    for(i=1;i<=NX;i++){
        for(j=1;j<=NX;j++){
            W(i,j) = 0.0;
        }
        W(i,i) = 1.0;
    }
}

double    q1 = 1e-8;
double    q2 = 1e-3;
double    q3 = 1e-3;
double    q4 = 1e-1;
double    r1 = 1e-4;
double    r2 = 1e-4;

void LibmFilter::makeBaseQ()
{
    int i,j;
    
    Q(1,1) = q1;
    Q(2,2) = q2;
    Q(3,3) = q3;
    Q(4,4) = q4;
    Q(5,5) = q1;
    Q(6,6) = q2;
    Q(7,7) = q3;
    Q(8,8) = q4;
}

void LibmFilter::makeBaseH()
{
    int i,j;
            
    for(i=1;i<=NY;i++)
        for(j=1;j<=NX;j++) H(i,j) = 0.0;

    H(1,1) = 1.0;

    H(2,3) = 1.0;    
    
    H(3,5) = 1.0;

    H(4,7) = 1.0;       
}


void LibmFilter::makeBaseV()
{
    int i,j;
    for(i=1;i<=NY;i++){
        for(j=1;j<=NY;j++){
            V(i,j) = 0.0;
        }
        V(i,i) = 1.0;
    }
}

void LibmFilter::makeBaseR()
{
    int i,j;
    
    R(1,1) = r1;
    R(2,2) = r2; 
    R(3,3) = r1;    
    R(4,4) = r2;    
}

 void LibmFilter_init(LibmFilter *p, double *x0, double *P0){
 	Vector x0v(NX);
     Matrix P0m(NX,NX);
 	int i,j;
        
 	for(i=0;i<NX;i++){
 		x0v(i+1) = x0[i];
 		//for(j=0;j<NX;j++)	P0m(1+i,1+j) = P0[NX*i + j];
		P0m(1+i,1+i) = P0[i];
 	}
 
 	p->init(x0v, P0m);

	p->Initialized = 1;
 }
 
 void LibmFilter_step(LibmFilter *p, double *input, double *meas, double *est){
 	Vector u(NU),m(NY),xv(NX);
 	int i,j;
 
 	for(i=0;i<NU;i++) u(1+i) = input[i];
    for(i=0;i<NY;i++) m(1+i) = meas[i];
        
 	p->step(u,m);
 
 	xv = p->getX();
 
 	for(i=0;i<NX;i++)	est[i] = xv(1+i);
 }