/*****************************************************************************/
/*
  animate.c: animate a data file.
*/
/*****************************************************************************/
#include "sdfast/sdlib.h"
#include "sdfast/SDModel.h"
#include "sdfast/hum1.h"

void hum1::sduforce( double t, double *q, double *u )
{
}

extern "C"{

/* This version flips X and angles to get lighting right */
#define XFLIP -
#define XXFLIP 

//#define DISPLAYVIRTUAL

#define DISPLAYPUSH

#ifdef DISPLAYPUSH
#define PUSHHEIGHT	1.2
#define PUSHTIME 1.0
#define PUSHDIRX	1
#define PUSHDIRY	0
#endif

/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#else
#include <time.h>
#endif

#include "matrix3.h"

#include "drawstuff.h" /* Graphics stuff */
#include "drawstuff-cga.h" /* CGA stuff to make things clearer */

#include "../useful/mrdplot/mrdplot.h"
#include "main.h"
#include "main2.h"

hum1 model;
SDModel *m = &model;

char joint_names[][20]= {
  {"L_HAA"},
  {"L_HFE"},
  {"L_KFE"},
  {"L_AR"},
  
  {"R_HAA"},
  {"R_HFE"},
  {"R_KFE"},
  {"R_AR"},
};


/*****************************************************************************/

#define TORSO_LENGTH (0.716711)     // length of link
#define TORSO_WIDTH (0.05f)    // width of link

#define PELVIS_LENGTH (2*0.0889)     // length of link
#define PELVIS_WIDTH (0.05f)    // width of link

#define THIGH_LENGTH (0.391794) // length of link
#define THIGH_WIDTH (0.05f)    // width of link

#define CALF_LENGTH (0.4171) // length of link
#define CALF_WIDTH (0.05f)    // width of link

#define FOOT_LENGTH (0.20)
#define FOOT_WIDTH (0.10)
#define FOOT_HEIGHT (0.025)

/*****************************************************************************/

SIM sim;
SIM *s = &sim;

MRDPLOT_DATA *d;

#define MAX_CHANNELS 100

int channels[MAX_CHANNELS];
int state_channels[MAX_CHANNELS];
int virt_state_channels[MAX_CHANNELS];

double start_time = 0;
double data_start_time = 0;

/*****************************************************************************/
/************************************************************************/

#ifdef WIN32
double get_time()
{
  return (double) timeGetTime()/1000.0;
}
#else
double get_time()
{
  struct timespec the_tp;

  clock_gettime( CLOCK_MONOTONIC, &the_tp );
  return ((double) (the_tp.tv_sec)) + 1.0e-9*the_tp.tv_nsec;
}
#endif

static void forward_kinematics()
{
  int i, j;
  // double zero_offset[XYZ] = { 0.0, 0.0, 0.0 };
  double torso_top_offset[XYZ]; // torso top relative to torso COM
  double torso_bottom_offset[XYZ];  // torso bottom relative to torso COM
  double hip_offset[LR][XYZ]; // hip relative to thigh COM
  double knee_offset[LR][XYZ]; // knee relative to calf COM
  double ankle_offset[LR][XYZ]; // foot relative to calf COM
  double dircos[3][3];

  // get position
  s->root_position_true[XX] = s->sdfast_state[XX];
  s->root_position_true[YY] = s->sdfast_state[YY];
  s->root_position_true[ZZ] = s->sdfast_state[ZZ];

  // reverse order from sdfast's quaternion to q that starts with cos() term
  //s->root_q_true[0] = s->sdfast_state[S_Q3];
  //s->root_q_true[1] = s->sdfast_state[S_Q0];
  //s->root_q_true[2] = s->sdfast_state[S_Q1];
  //s->root_q_true[3] = s->sdfast_state[S_Q2];

  //// get velocity
  //s->root_velocity_true[XX] = s->sdfast_state[m->NQ()+0];
  //s->root_velocity_true[YY] = s->sdfast_state[m->NQ()+1];
  //s->root_velocity_true[ZZ] = s->sdfast_state[m->NQ()+2];

  //// get angular velocity
  //s->root_w_body_true[XX] = s->sdfast_state[m->NQ()+3];
  //s->root_w_body_true[YY] = s->sdfast_state[m->NQ()+4];
  //s->root_w_body_true[ZZ] = s->sdfast_state[m->NQ()+5];

  // track quaternion norm errors
  //s->root_q_true_norm = normalize_q( s->root_q_true, NULL ) - 1.0;

  // quaternion to rotation matrix
  //q_to_r( s->root_q_true, s->root_r_true );

    // rotate angular velocity in body coordinates to inertial coordinates
  //multiply_m_v( s->root_r_true, s->root_w_body_true, s->root_w_world_true );

  // AFE, AAA set by compute_foot_orientation

  for( i = 0; i < N_LOCATIONS; i++ )
    for( j = XX; j <= ZZ; j++ )
      s->locations_true[i][j] = 0;

  for( i = 0; i < 3; i++ )
    {
      torso_top_offset[i] = 0;
      torso_bottom_offset[i] = 0;
      for( j = LEFT; j <= RIGHT; j++ )
	{
	  hip_offset[j][i] = 0;
	  knee_offset[j][i] = 0;
	  ankle_offset[j][i] = 0;
	  s->foot_offset[j][i] = 0;
	}
    }
  // see dynamics-impact.c for better formulation
  torso_top_offset[ZZ] = 0.2;
  torso_bottom_offset[ZZ] = -0.516711;
  // These should be L_HIP and R_HIP
  m->sdgetbtj( B_L_UTHIGH, &(hip_offset[LEFT][XX]) );
  m->sdgetbtj( B_R_UTHIGH, &(hip_offset[RIGHT][XX]) );
  m->sdgetbtj( B_L_UCALF, &(knee_offset[LEFT][XX]) );
  m->sdgetbtj( B_R_UCALF, &(knee_offset[RIGHT][XX]) );
  for ( i = 0; i < 3; i++ )
    {
      ankle_offset[LEFT][i] = 0.0;
      s->foot_offset[LEFT][i] = 0.0;
      ankle_offset[RIGHT][i] = 0.0;
      s->foot_offset[RIGHT][i] = 0.0;
    }
  // Fudge ankle and foot offsets for artistic reasons
  ankle_offset[LEFT][XX] = -0.03896;
  ankle_offset[LEFT][YY] = -0.09783+0.0889;
  for( i = 0; i < 3; i++ )
    {
      s->foot_offset[LEFT][i] = ankle_offset[LEFT][i];
      ankle_offset[RIGHT][i] = ankle_offset[LEFT][i];
      s->foot_offset[RIGHT][i] = s->foot_offset[LEFT][i];
    }
  ankle_offset[RIGHT][YY] = -ankle_offset[LEFT][YY];
  s->foot_offset[RIGHT][YY] = -s->foot_offset[LEFT][YY];

  m->sdpos( B_PELVIS, torso_top_offset, &(s->locations_true[P_TORSO_TOP][0]) );    
  m->sdpos( B_PELVIS, torso_bottom_offset, &(s->locations_true[P_TORSO_BOTTOM][0]) );
  m->sdpos( B_L_UTHIGH, &(hip_offset[LEFT][0]), &(s->locations_true[P_L_HIP][0]) );
  m->sdpos( B_R_UTHIGH, &(hip_offset[RIGHT][0]), &(s->locations_true[P_R_HIP][0]) );
  m->sdpos( B_L_UCALF, &(knee_offset[LEFT][0]), &(s->locations_true[P_L_KNEE][0]) );
  m->sdpos( B_R_UCALF, &(knee_offset[RIGHT][0]), &(s->locations_true[P_R_KNEE][0]) );    
  m->sdpos( B_L_FOOT, &(ankle_offset[LEFT][0]), &(s->locations_true[P_L_ANKLE][0]) );    
  m->sdpos( B_R_FOOT, &(ankle_offset[RIGHT][0]), &(s->locations_true[P_R_ANKLE][0]) );
  m->sdpos( B_L_FOOT, &(s->foot_offset[LEFT][0]), &(s->foot_true[LEFT][0]) );    
  m->sdpos( B_R_FOOT, &(s->foot_offset[RIGHT][0]), &(s->foot_true[RIGHT][0]) );    

  m->sdorient( B_PELVIS, dircos );
  sddc2quat( dircos, &(s->orientations_true[O_TORSO_Q][1]),
	     &(s->orientations_true[O_TORSO_Q][2]),
	     &(s->orientations_true[O_TORSO_Q][3]),
	     &(s->orientations_true[O_TORSO_Q][0]) );
  sddc2ang(dircos,&s->root_euler_true[0],&s->root_euler_true[1],&s->root_euler_true[2]);
  m->sdorient( B_L_UTHIGH, dircos );
  sddc2quat( dircos, &(s->orientations_true[O_L_THIGH_Q][1]),
	     &(s->orientations_true[O_L_THIGH_Q][2]),
	     &(s->orientations_true[O_L_THIGH_Q][3]),
	     &(s->orientations_true[O_L_THIGH_Q][0]) );
  m->sdorient( B_L_UCALF, dircos );
  sddc2quat( dircos, &(s->orientations_true[O_L_CALF_Q][1]),
	     &(s->orientations_true[O_L_CALF_Q][2]),
	     &(s->orientations_true[O_L_CALF_Q][3]),
	     &(s->orientations_true[O_L_CALF_Q][0]) );
  m->sdorient( B_R_UTHIGH, dircos );
  sddc2quat( dircos, &(s->orientations_true[O_R_THIGH_Q][1]),
	     &(s->orientations_true[O_R_THIGH_Q][2]),
	     &(s->orientations_true[O_R_THIGH_Q][3]),
	     &(s->orientations_true[O_R_THIGH_Q][0]) );
  m->sdorient( B_R_UCALF, dircos );
  sddc2quat( dircos, &(s->orientations_true[O_R_CALF_Q][1]),
	     &(s->orientations_true[O_R_CALF_Q][2]),
	     &(s->orientations_true[O_R_CALF_Q][3]),
	     &(s->orientations_true[O_R_CALF_Q][0]) );
}


/*****************************************************************************/

//double lookup_value( MRDPLOT_DATA *d, int point, int channel )
//{
//  if ( channels[channel] < 0 || channels[channel] >= d->n_channels )
//    {
//      fprintf( stderr, "Bad channel: %d %d\n", channel, channels[channel] );
//      exit( -1 );
//    }
//  return d->data[point*d->n_channels + channels[channel]];
//}

double lookup_value2( MRDPLOT_DATA *d, int point, int channel )
{
  if ( state_channels[channel] < 0 || state_channels[channel] >= d->n_channels )
    {
      fprintf( stderr, "Bad channel: %d %d\n", channel, state_channels[channel] );
      exit( -1 );
    }
  return d->data[point*d->n_channels + state_channels[channel]];
}

double lookup_value3( MRDPLOT_DATA *d, int point, int channel )
{
  if ( virt_state_channels[channel] < 0 || virt_state_channels[channel] >= d->n_channels )
    {
      fprintf( stderr, "Bad channel: %d %d\n", channel, virt_state_channels[channel] );
      exit( -1 );
    }
  return d->data[point*d->n_channels + virt_state_channels[channel]];
}

/********************************************************************/
// start(): set things up

static void start()
{
  // set up view point
  /*
  static float xyz[3] = {0.0f,1.4f,0.8f};
  static float hpr[3] = {-90.0000f,0.0000f,0.0000f};
  */
	
#ifdef DISPLAYVIRTUAL
  static float xyz[3] = {1.3f,0.3f,0.8f};
#else
  static float xyz[3] = {1.3f,0.0f,0.8f};
#endif
  static float hpr[3] = {180.0000f,0.0000f,0.0000f};
  dsSetViewpoint (xyz,hpr);
}

/********************************************************************/
// called when a key pressed

static void command (int cmd)
{
  // don't handle user input yet.
  dsPrint ("received command %d (`%c')\n",cmd,cmd);
}

/*****************************************************************************/

void get_center2( double*p1, double*p2, float *center )
{
  int i;

  for( i = XX; i <= ZZ; i++ )
    center[i] = 0.5*(p1[i]+p2[i]);
}

/*****************************************************************************/


void get_r2( double *q, float *R )
{
  int i;
  double r[3][3];
  q_to_r( q, r );

  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  // R[0] = R[5] = R[10] = 1.0;
  for ( i = 0; i < 3; i++ )
    {
      R[i] = r[0][i];
      R[i+4] = r[1][i];
      R[i+8] = r[2][i];
    }
  // exit( -1 );
}

/*****************************************************************************/

#define VIRT_OFFSET	0.5

static void display2 (int pause)
{
  static int index = 0;
  float f;
  float center[3];
  float R[12];
  int i;
  float calf_sides[3] = { CALF_WIDTH, CALF_WIDTH, CALF_LENGTH };
  float thigh_sides[3] = { THIGH_WIDTH, THIGH_WIDTH, THIGH_LENGTH };
  float torso_sides[3] = { TORSO_WIDTH, TORSO_WIDTH, TORSO_LENGTH };
  float pelvis_sides[3] = { PELVIS_WIDTH, PELVIS_LENGTH, PELVIS_WIDTH };
  float foot_sides[3] = { FOOT_LENGTH, FOOT_WIDTH, FOOT_HEIGHT };
  float marker[3];
  float data_point_time;
  double the_time;

  // state indices in state_channels (TIME = 0)
  for(i=0;i<7+N_JOINTS-4;i++)	  s->sdfast_state[i] = lookup_value2( d, index, i+1 );
  m->sdstate(0.0,&s->sdfast_state[0],&s->sdfast_state[7+N_JOINTS-4]);

  forward_kinematics();

  /* Torso */
  get_center2( s->locations_true[P_TORSO_TOP], s->locations_true[P_TORSO_BOTTOM], center );
  get_r2( s->orientations_true[O_TORSO_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, torso_sides );

  /* Pelvis */
  get_center2( s->locations_true[P_L_HIP], s->locations_true[P_R_HIP], center );
  get_r2( s->orientations_true[O_TORSO_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, pelvis_sides );

  /* Left Thigh */
  get_center2( s->locations_true[P_L_HIP], s->locations_true[P_L_KNEE], center );
  get_r2( s->orientations_true[O_L_THIGH_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,1,0);
  dsDrawBox( center, R, thigh_sides );

  /* Left Calf */
  get_center2( s->locations_true[P_L_KNEE], s->locations_true[P_L_ANKLE], center );
  get_r2( s->orientations_true[O_L_CALF_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,1,0);
  dsDrawBox( center, R, calf_sides );

  /* Right Thigh */
  get_center2( s->locations_true[P_R_HIP], s->locations_true[P_R_KNEE], center );
  get_r2( s->orientations_true[O_R_THIGH_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, thigh_sides );

  /* Right Calf */
  get_center2( s->locations_true[P_R_KNEE], s->locations_true[P_R_ANKLE], center );
  get_r2( s->orientations_true[O_R_CALF_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, calf_sides );

  /* Markers */
  // set up identity matrix
  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = R[5] = R[10] = 1.0;

  data_point_time = lookup_value2( d, index, 0 );

  marker[XX] = s->locations_true[P_TORSO_TOP][0];
  marker[YY] = s->locations_true[P_TORSO_TOP][1];
  marker[ZZ] = s->locations_true[P_TORSO_TOP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  marker[XX] = s->locations_true[P_TORSO_BOTTOM][0];
  marker[YY] = s->locations_true[P_TORSO_BOTTOM][1];
  marker[ZZ] = s->locations_true[P_TORSO_BOTTOM][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_HIP][0];
  marker[YY] = s->locations_true[P_L_HIP][1];
  marker[ZZ] = s->locations_true[P_L_HIP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_KNEE][0];
  marker[YY] = s->locations_true[P_L_KNEE][1];
  marker[ZZ] = s->locations_true[P_L_KNEE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_ANKLE][0];
  marker[YY] = s->locations_true[P_L_ANKLE][1];
  marker[ZZ] = s->locations_true[P_L_ANKLE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.02 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_HIP][0];
  marker[YY] = s->locations_true[P_R_HIP][1];
  marker[ZZ] = s->locations_true[P_R_HIP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_KNEE][0];
  marker[YY] = s->locations_true[P_R_KNEE][1];
  marker[ZZ] = s->locations_true[P_R_KNEE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_ANKLE][0];
  marker[YY] = s->locations_true[P_R_ANKLE][1];
  marker[ZZ] = s->locations_true[P_R_ANKLE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.02 );

  center[XX] = s->foot_true[LEFT][0];
  center[YY] = s->foot_true[LEFT][1];
  center[ZZ] = s->foot_true[LEFT][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawBox( center, R, foot_sides );

  center[XX] = s->foot_true[RIGHT][0];
  center[YY] = s->foot_true[RIGHT][1];
  center[ZZ] = s->foot_true[RIGHT][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawBox( center, R, foot_sides );


// virtual/kinematic model display
#ifdef DISPLAYVIRTUAL

  for(i=0;i<7+N_JOINTS-4;i++)	  s->sdfast_state[i] = lookup_value3( d, index, i+1 );
  s->sdfast_state[1] += VIRT_OFFSET;
  m->sdstate(0.0,&s->sdfast_state[0],&s->sdfast_state[7+N_JOINTS-4]);


  forward_kinematics();

  /* Torso */
  get_center2( s->locations_true[P_TORSO_TOP], s->locations_true[P_TORSO_BOTTOM], center );  
  get_r2( s->orientations_true[O_TORSO_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, torso_sides );

  /* Pelvis */
  get_center2( s->locations_true[P_L_HIP], s->locations_true[P_R_HIP], center );  
  get_r2( s->orientations_true[O_TORSO_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, pelvis_sides );

  /* Left Thigh */
  get_center2( s->locations_true[P_L_HIP], s->locations_true[P_L_KNEE], center );  
  get_r2( s->orientations_true[O_L_THIGH_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,1,0);
  dsDrawBox( center, R, thigh_sides );

  /* Left Calf */
  get_center2( s->locations_true[P_L_KNEE], s->locations_true[P_L_ANKLE], center );  
  get_r2( s->orientations_true[O_L_CALF_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,1,0);
  dsDrawBox( center, R, calf_sides );

  /* Right Thigh */
  get_center2( s->locations_true[P_R_HIP], s->locations_true[P_R_KNEE], center );  
  get_r2( s->orientations_true[O_R_THIGH_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, thigh_sides );

  /* Right Calf */
  get_center2( s->locations_true[P_R_KNEE], s->locations_true[P_R_ANKLE], center );  
  get_r2( s->orientations_true[O_R_CALF_Q], R );
  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, calf_sides );

  /* Markers */
  // set up identity matrix
  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = R[5] = R[10] = 1.0;

  data_point_time = lookup_value2( d, index, 0 );

  marker[XX] = s->locations_true[P_TORSO_TOP][0];
  marker[YY] = s->locations_true[P_TORSO_TOP][1];
  marker[ZZ] = s->locations_true[P_TORSO_TOP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  marker[XX] = s->locations_true[P_TORSO_BOTTOM][0];
  marker[YY] = s->locations_true[P_TORSO_BOTTOM][1];
  marker[ZZ] = s->locations_true[P_TORSO_BOTTOM][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_HIP][0];
  marker[YY] = s->locations_true[P_L_HIP][1];
  marker[ZZ] = s->locations_true[P_L_HIP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_KNEE][0];
  marker[YY] = s->locations_true[P_L_KNEE][1];
  marker[ZZ] = s->locations_true[P_L_KNEE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_L_ANKLE][0];
  marker[YY] = s->locations_true[P_L_ANKLE][1];
  marker[ZZ] = s->locations_true[P_L_ANKLE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.02 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_HIP][0];
  marker[YY] = s->locations_true[P_R_HIP][1];
  marker[ZZ] = s->locations_true[P_R_HIP][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_KNEE][0];
  marker[YY] = s->locations_true[P_R_KNEE][1];
  marker[ZZ] = s->locations_true[P_R_KNEE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.05 );

  /* Markers */
  marker[XX] = s->locations_true[P_R_ANKLE][0];
  marker[YY] = s->locations_true[P_R_ANKLE][1];
  marker[ZZ] = s->locations_true[P_R_ANKLE][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawSphere( marker, R, 0.02 );

  center[XX] = s->foot_true[LEFT][0];
  center[YY] = s->foot_true[LEFT][1];
  center[ZZ] = s->foot_true[LEFT][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawBox( center, R, foot_sides );

  center[XX] = s->foot_true[RIGHT][0];
  center[YY] = s->foot_true[RIGHT][1];
  center[ZZ] = s->foot_true[RIGHT][2];
  dsSetTexture (DS_NONE);
  dsSetColor (1,1,1);
  dsDrawBox( center, R, foot_sides );

#endif

#ifdef DISPLAYPUSH
	if((data_point_time>(PUSHTIME-1.0)) && (data_point_time<=PUSHTIME)){
	  marker[XX] = -PUSHDIRX*1.0 + 1.0*(data_point_time-PUSHTIME+1.0)*PUSHDIRX;
	  marker[YY] = -PUSHDIRY*1.0 + 1.0*(data_point_time-PUSHTIME+1.0)*PUSHDIRY;
	  marker[ZZ] = PUSHHEIGHT;
	  dsSetTexture (DS_NONE);
	  dsSetColor (1,1,1);
	  dsDrawSphere( marker, R, 0.08 );
	}
#endif


  /* Delay */
#ifdef WIN32
  for( ; ; )
    {
      the_time = get_time();
      if ( the_time - start_time > data_point_time - data_start_time )
	break;
      for( f = 0.0; f < 100000.0; f += 1.0 )
	;
    }
#else
  for( ; ; )
    {
      the_time = get_time();
      if ( the_time - start_time > data_point_time - data_start_time )
	break;
      for( f = 0.0; f < 100000.0; f += 1.0 )
	;
    }
#endif

  index += 33;
  if ( index > d->n_points )
    {
      index = 0;
#ifdef WIN32
      the_time = get_time();
      printf( "time: %g\n", the_time - start_time );
      start_time = the_time;
#else
      the_time = get_time();
      printf( "time: %g\n", the_time - start_time );
      start_time = the_time;
#endif
    }
}


/*****************************************************************************/
/********************************************************************/

int main (int argc, char **argv)
{
  char *filename;
  int i;
  dsFunctions fn; 
  char buff[20];

  if ( argc > 1 )
    filename = strdup( argv[1] );
  else
    filename = last_data();
  
  printf( "Reading: %s\n", filename );

  d = read_mrdplot( filename );

  for( i = 0; i < N_STATE+1; i++ )
    state_channels[i] = -1;

  state_channels[0] = find_channel("time",d);
  state_channels[1] = find_channel( "x", d );
  state_channels[2] = find_channel( "y", d );
  state_channels[3] = find_channel( "z", d );
  state_channels[4] = find_channel( "a_q0", d );
  state_channels[5] = find_channel( "a_q1", d );
  state_channels[6] = find_channel( "a_q2", d );
  for(i=0;i<N_U-6;i++){
	  sprintf(buff,"%s_th",joint_names[i]);
	  state_channels[7+i] = find_channel( buff, d );
  }
  state_channels[1+N_U] = find_channel( "a_q3", d );

  virt_state_channels[0] = find_channel( "time", d );
  virt_state_channels[1] = find_channel( "virt_x", d );
  virt_state_channels[2] = find_channel( "virt_y", d );
  virt_state_channels[3] = find_channel( "virt_z", d );
  virt_state_channels[4] = find_channel( "virt_q0", d );
  virt_state_channels[5] = find_channel( "virt_q1", d );
  virt_state_channels[6] = find_channel( "virt_q2", d );
  for(i=0;i<N_U-6;i++){
	  sprintf(buff,"virt_%s_th",joint_names[i]);
	  virt_state_channels[7+i] = find_channel( buff, d );
  }
  virt_state_channels[1+N_U] = find_channel( "virt_q3", d );


  //for( i = 0; i < N_CHANNELS; i++ )
  //  printf( "%d %d\n", i, channels[i] );

 // for( i = 0; i < 1+N_U; i++ )
    //printf( "%d %d\n", i, state_channels[i] );

  // setup pointers to drawstuff callback functions
  fn.version = DS_VERSION; 
  fn.start = &start; 
  fn.step = &display2; 
  fn.command = &command; 
  fn.stop = 0; 
#ifdef WIN32
  // fn.path_to_textures = "C:/cga/hum1/useful/drawstuff-windows/textures";
  fn.path_to_textures = "../useful/drawstuff-windows/textures";
#else
  // fn.path_to_textures = "/home/cga/projects/useful/drawstuff/textures";
  fn.path_to_textures = "../useful/drawstuff-linux/textures";
#endif

#ifdef WIN32
  start_time = get_time();
#else
  start_time = get_time();
#endif

  m->sdinit();

  data_start_time = lookup_value2( d, 0, 0 );

  // do display
#ifdef DISPLAYVIRTUAL
  dsSimulationLoop( argc, argv, /* command line arguments */
                    400, 500, /* window size */
                    &fn ); /* callback info */
#else
  dsSimulationLoop( argc, argv, /* command line arguments */
                    720, 480, /* window size */
                    &fn ); /* callback info */
#endif
  return 0;
}

/********************************************************************/
}
