#include "ben_common.h"
#include "ben_controller.h"
#include "libmquad.h"
#include "gsl_util.h"
#include "hum2.h"
#include "SDModel.h"
#include "SDModel_common.h"
#include "sdlib.h"
#include "primus_common.h"

#include "state_estimator.h"
#include "state_machine.h"

#include "inverse_kinematics_quad.h"

extern "C"{
	#include <math.h>


static hum2 control_model, des_model;
static SDModel *m = &control_model;
static SDModel *m2 = &des_model;
static double state[N_STATE]={0.0};


void my_init_controller(PRIMUS_t *p){
	int i;
	for(i=0;i<3;i++){
	  p->footd_d[LEFT][i] = 0.0;
	  p->footdd_d[LEFT][i] = 0.0;
	  p->footd_d[RIGHT][i] = 0.0;
	  p->footdd_d[RIGHT][i] = 0.0;
	  p->comd_d[i] = 0.0;
	  p->comdd_d[i] = 0.0;
	}

	p->controller_state=STATE_HOME;
	p->cstate_elapsed_time=0.0;

	p->total_mass = 78.2875;
	
	m->sdinit();
	m2->sdinit();
}


static void com_controller(PRIMUS_t *s){
	double Kx=5.0;
	double Ky=5.0;
	double Kz=10.0;

#ifdef USE_STATE_ESTIMATOR
	s->comdd_d[0] = s->comdd_ff[0] + Kx*(s->com_d[0]-s->com_est[0]) + 4*sqrt(Kx)*(s->comd_d[0]-s->comd_est[0]);
	s->comdd_d[1] = s->comdd_ff[1] + Ky*(s->com_d[1]-s->com_est[1]) + 4*sqrt(Ky)*(s->comd_d[1]-s->comd_est[1]);
#else
	s->comdd_d[0] = s->comdd_ff[0] + Kx*(s->com_d[0]-s->com[0]) + 4*sqrt(Kx)*(s->comd_d[0]-s->comd[0]);
	s->comdd_d[1] = s->comdd_ff[1] + Ky*(s->com_d[1]-s->com[1]) + 4*sqrt(Ky)*(s->comd_d[1]-s->comd[1]);
#endif
	s->comdd_d[2] = Kz*(s->com_d[2]-s->com[2]);
}


static void solve_feet_forces_quad(PRIMUS_t *s){
	double F[12];
	int i;
	
	double params[5] = {78.2875,1.0,0.1,0.05,0.3};
	double w[12] = {10.0,10.0,0.1,
		            1.0,1.0,1.0,
					10.0,10.0,0.1,
					1.0,1.0,1.0};
	double libm_state[8];

#ifdef USE_STATE_ESTIMATOR
	libm_state[0] = s->com_est[0];
	libm_state[1] = s->com_est[1];
	libm_state[2] = s->comd_est[0];
	libm_state[3] = s->comd_est[1];
#else
	libm_state[0] = s->com[0];
	libm_state[1] = s->com[1];
	libm_state[2] = s->comd[0];
	libm_state[3] = s->comd[1];

#endif
	libm_state[4] = s->foot[LEFT][0];
	libm_state[5] = s->foot[LEFT][1];
	libm_state[6] = s->foot[RIGHT][0];
	libm_state[7] = s->foot[RIGHT][1];

	//=== Calculate forces using QP Optimization
	libmquad(libm_state, s->comdd_d, s->task_force, s->task_torque, params, s->contact_state, w, F);

	for(i=0;i<6;i++){
		s->foot_force_des[0][i] = F[i];
		s->foot_force_des[1][i] = F[6+i];
	}	
}


static void run_inverse_dynamics(PRIMUS_t *s){
	static int firsttime=1;
	//Primus *p = new Primus(m);
	double tau[N_U];
	double PL[6],PR[6],JL[N_U*6],JR[N_U*6],M[N_U][N_U];	
	double PH[6],JH[N_U*6];
	double zero[3]={0.0};
	int i,j;
	static int count=0;
	double A[N_U+12+N_U+N_J][N_U+N_J]={0.0};
	double b[N_U+12+N_U+N_J],x[N_U+N_J];
	static double JL2[N_U*6]={0.0},JR2[N_U*6]={0.0};

	double footdd_d_2[2][6] = {0.0};

	primus_feet(m,PL,JL,PR,JR);

	body_position(m,NULL,0,zero,PH,JH);

	m->sdequivht(tau);

	m->sdmassmat((double*)M);

	for(i=0;i<N_U;i++){
		for(j=0;j<N_U;j++){
			A[i][j] = M[i][j];
		}
	}

	for(i=0;i<6;i++){
		for(j=0;j<N_U;j++){
			A[N_U+i][j] = JL[6*j + i];
			A[N_U+6+i][j] = JR[6*j + i];
		}

		// should be -Jdot*qdot
		b[N_U+i] = 0.0;
		b[N_U+6+i] = 0.0;
		if (count>0){
			for(j=0;j<N_U;j++){
				//b[N_U+0+i] -= (JL2[6*j+i] - JL[6*j+i])/0.001*state[m->NQ()+j];
				//b[N_U+6+i] -= (JR2[6*j+i] - JR[6*j+i])/0.001*state[m->NQ()+j];
			}
		}
	}

	if(s->contact_state==CONTACT_RSP){ 
		footdd_d_2[LEFT][0] = 0*s->footdd_d[LEFT][0] + 100.0*(s->foot_d[LEFT][0]-s->foot[LEFT][0])+5.0*(s->footd_d[LEFT][0]-0*s->footd[LEFT][0]);
		footdd_d_2[LEFT][1] = 0*s->footdd_d[LEFT][1] + 100.0*(s->foot_d[LEFT][1]-s->foot[LEFT][1])+5.0*(s->footd_d[LEFT][1]-0*s->footd[LEFT][1]);
		footdd_d_2[LEFT][2] = s->footdd_d[LEFT][2] + 300.0*(s->foot_d[LEFT][2]-s->foot[LEFT][2])+10.0*(s->footd_d[LEFT][2]-0*s->footd[LEFT][2]);
	}
	if(s->contact_state==CONTACT_LSP){
		footdd_d_2[RIGHT][0] = 0*s->footdd_d[RIGHT][0] + 100.0*(s->foot_d[RIGHT][0]-s->foot[RIGHT][0])+5.0*(s->footd_d[RIGHT][0]-0*s->footd[RIGHT][0]);
		footdd_d_2[RIGHT][1] = 0*s->footdd_d[RIGHT][1] + 100.0*(s->foot_d[RIGHT][1]-s->foot[RIGHT][1])+5.0*(s->footd_d[RIGHT][1]-0*s->footd[RIGHT][1]);
		footdd_d_2[RIGHT][2] = s->footdd_d[RIGHT][2] + 300.0*(s->foot_d[RIGHT][2]-s->foot[RIGHT][2])+10.0*(s->footd_d[RIGHT][2]-0*s->footd[RIGHT][2]);
	}

	for(i=0;i<3;i++){
		b[N_U+i+6*0] += footdd_d_2[LEFT][i];
		b[N_U+i+6*1] += footdd_d_2[RIGHT][i];
	}

	for(i=0;i<N_J;i++)	A[6+i][N_U+i] = -1.0;

	for(i=0;i<N_U;i++){
		A[N_U+12+i][i] = 1.0e-4;
		b[N_U+12+i] = 0.0;
	}

	for(i=0;i<N_J;i++){
		A[N_U+12+N_U+i][N_U+i] = 1.0e-5;
		b[N_U+12+N_U+i] = 0.0;
	}

#ifdef USE_MULTITHREADING_ID

	if (firsttime){
		firsttime=0;
		// create mutex and events
		inversedynData.hMutex = CreateMutex(NULL,0,NULL);
		// start the thread
		inversedynData.hThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)inversedynThread,&inversedynData,0,&inversedynData.lpThreadID);
		SetThreadPriority(inversedynData.hThread,THREAD_PRIORITY_TIME_CRITICAL);
	}else{
		WaitForSingleObject(inversedynData.Mutex,INFINITE);
			// copy A to inversedynData.A
		ReleaseMutex(inversedynData.hMutex);
	}

#endif
	

	for(i=0;i<N_U;i++)	{

		b[i] = tau[i];
		
		if (s->foot_force[0][2]>10)
		for(j=0;j<6;j++)	b[i] += JL[6*i + j]*s->foot_force_des[LEFT][j];
		
		if (s->foot_force[1][2]>10)
		for(j=0;j<6;j++)	b[i] += JR[6*i + j]*s->foot_force_des[RIGHT][j];

	if(s->contact_state==CONTACT_RSP){
	    b[i] += -JL[6*i + 0]*30*footdd_d_2[LEFT][0];
	    b[i] += -JL[6*i + 1]*30*footdd_d_2[LEFT][1];
	    b[i] += -JL[6*i + 2]*100*footdd_d_2[LEFT][2];
	}
	if(s->contact_state==CONTACT_LSP){
	    b[i] += -JR[6*i + 0]*30*footdd_d_2[RIGHT][0];
	    b[i] += -JR[6*i + 1]*30*footdd_d_2[RIGHT][1];
	    b[i] += -JR[6*i + 2]*100*footdd_d_2[RIGHT][2];
	}

		// apply virtual torque to torso
		for(j=0;j<3;j++)	b[i] += JH[6*i + 3+j]*s->torso_torque[j];
	}	

	gsl_pseudosolve_vector((double*)A, b, 0, N_U+12+N_U+N_J, N_U+N_J, x);

	for(i=0;i<N_J;i++)	s->torque_ff[i] = x[N_U+i];

	for(i=0;i<6;i++){
		for(j=0;j<N_U;j++){
			JL2[6*j+i] = JL[6*j+i];
			JR2[6*j+i] = JR[6*j+i];
		}
	}

	count++;

}


void my_controller(PRIMUS_t *s){
	int i;
	double lin[3],KE;

  my_state_estimator(s);
  
  set_state(m,s->state);

  run_state_machine( s );
  
//#ifdef USE_STATE_ESTIMATOR
//  s->cop_d[0] = s->com_est[0] - s->comdd_d[0]/9.81;
//  s->cop_d[1] = s->com_est[1] - s->comdd_d[1]/9.81;
//#else
//  s->cop_d[0] = s->com[0] - s->comdd_ff[0]/9.81;
//  s->cop_d[1] = s->com[1] - s->comdd_ff[1]/9.81;
//#endif

  //s->com_d[2] = 0.8 + 0.1*sin(6*s->time);
  //s->comd_d[2] = 6*0.1*cos(6*s->time);

  if (0){
	  run_inverse_kinematics_quad( s );
	  for(i=0;i<N_Q+N_U;i++)	s->state_des[i] = s->state_ik[i];
	  //for(i=0;i<N_J;i++)	s->state_des[6+i] = s->state_ik[6+i];
	  //for(i=0;i<N_J;i++)	s->state_des[N_Q+6+i] = s->state_ik[N_Q+6+i];
  }

	//for(i=0;i<N_JOINTS;i++)	s->joints_d[i] = s->des_state[6+i];
	//for(i=0;i<N_JOINTS;i++)	s->jointsd_d[i] = s->des_state[N_Q+6+i];

	  //if (contact_state==DSP){
		s->torso_torque[0] = -200.0*s->torso[0] - 0.3*s->torsod[0];
		s->torso_torque[1] = -200.0*s->torso[1] - 0.3*s->torsod[1];
		s->torso_torque[2] = -100.0*s->torso[2] - 0.1*s->torsod[2]; 
	  //}

	m->sdmom(lin,s->angmom,&KE);

	s->task_torque[0] = s->torso_torque[0]-30.0*s->angmom[0];
	s->task_torque[1] = s->torso_torque[1]-30.0*s->angmom[1];
	s->task_torque[2] = s->torso_torque[2]-10.0*s->angmom[2];

	if (s->controller_state!=STATE_LAUNCH){
	 // // calculate desired COM acceleration
	  com_controller(s);


	 // // solve for feet forces
	  solve_feet_forces_quad(s);

	 // // solve inverse dynamics
	  run_inverse_dynamics(s);
	}


}

}
