/*****************************************************************************/
/*
  controller.c: control strategy.
*/
/*****************************************************************************/

#include "ben_common.h"
#include "ben_controller.h"

extern "C"{


#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "sdlib.h"

#include "../useful/trajectory/trajectory.h"
#include "main.h"
#include "main2.h"




/*****************************************************************************/


#define UNKNOWN_STATE 99

/* Joint servo modes */
#define PID_MODE 0
#define TORQUE_MODE 1

/* Trajectory Generation Modes */
#define NO_TRAJECTORY 0
#define USE_TRAJECTORY 1


/*****************************************************************************/
/* Control variables */

DOFS desired_joint_trajectory[N_JOINTS];
DOFS current_joint_desireds[N_JOINTS];


/*****************************************************************************/
/*****************************************************************************/

void
init_controller_parameters( SIM *s )
{
  int i, j;

  s->duration = 5.0;
  // s->duration = 0.0001;
  s->time_step = 0.001;

  s->controller_print = 0;

  s->in_air_not_allowed = 1;

  // PATTERN VARIABLES

  // obsolete
  s->foot_width = 0.213; // distance between feet in stance.

  s->com_height_zero = 0.9763;


  for( i = 0; i < N_JOINTS; i++ )
    for( j = 0; j < N_JOINTS; j++ )
      {
	s->pos_gains[i][j] = 0;
	s->vel_gains[i][j] = 0;
	s->int_gains[i][j] = 0;
      }

  // merge these
  //s->torque_max_hfe = 226;
  //s->torque_max_haa = 107;
  //s->torque_max_hfr = 42;
  //s->torque_max_kfe = 293;
  //s->torque_max_ar = 74;
  //s->torque_max_afe = 202;
  //s->torque_max_aaa = 167;

  s->torque_max_hfe = 200;
  s->torque_max_haa = 200;
  s->torque_max_hfr = 200;
  s->torque_max_kfe = 200;
  s->torque_max_ar = 200;
  s->torque_max_afe = 200;
  s->torque_max_aaa = 200;
  for( i = 0; i < N_JOINTS; i++ )
    {
      s->controls_min[i] = -200;
      s->controls_max[i] = 200;
    }

  s->rand_scale = 0.0;
  s->rand_seed = 1;



  //init_trajectory(com_traj,2);
  //init_trajectory(foot_traj,3);
}


/*****************************************************************************/

void
set_double_support_stance_gains( SIM *s )
{
  int i, j;

  for( i = 0; i < N_JOINTS; i++ )
    {
      s->servo_mode[i] = PID_MODE;
      for( j = 0; j < N_JOINTS; j++ )
	{
	  s->pos_gains[i][j] = 0;
	  s->vel_gains[i][j] = 0;
	}
    }

  s->pos_gains[L_HAA][L_HAA] = 20.0;
  s->vel_gains[L_HAA][L_HAA] = 5.0;
  s->pos_gains[L_HFE][L_HFE] = 20.0;
  s->vel_gains[L_HFE][L_HFE] = 5.0;
  s->int_gains[L_HFE][L_HFE] = 0.0;
  s->pos_gains[L_KFE][L_KFE] = 20.0;
  s->vel_gains[L_KFE][L_KFE] = 5.0;
  s->int_gains[L_KFE][L_KFE] = 0.0;
  s->pos_gains[L_AR][L_AR] = 20.0;
  s->vel_gains[L_AR][L_AR] = 5.0;
  s->int_gains[L_AR][L_AR] = 0.0;
  s->pos_gains[L_AFE][L_AFE] = 20.0;
  s->vel_gains[L_AFE][L_AFE] = 5.0;
  s->int_gains[L_AFE][L_AFE] = 0.0;
  s->pos_gains[L_AAA][L_AAA] = 20.0;
  s->vel_gains[L_AAA][L_AAA] = 5.0;

  s->pos_gains[R_HAA][R_HAA] = 20.0;
  s->vel_gains[R_HAA][R_HAA] = 5.0;
  s->pos_gains[R_HFE][R_HFE] = 20.0;
  s->vel_gains[R_HFE][R_HFE] = 5.0;
  s->int_gains[R_HFE][R_HFE] = 0.0;
  s->pos_gains[R_KFE][R_KFE] = 20.0;
  s->vel_gains[R_KFE][R_KFE] = 5.0;
  s->int_gains[R_KFE][R_KFE] = 0.0;
  s->pos_gains[R_AR][R_AR] = 20.0;
  s->vel_gains[R_AR][R_AR] = 5.0;
  s->int_gains[R_AR][R_AR] = 0.0;
  s->pos_gains[R_AFE][R_AFE] = 20.0;
  s->vel_gains[R_AFE][R_AFE] = 5.0;
  s->int_gains[R_AFE][R_AFE] = 0.0;
  s->pos_gains[R_AAA][R_AAA] = 20.0; 
  s->vel_gains[R_AAA][R_AAA] = 5.0;


}

/*****************************************************************************/
/* call this many times to restart a simulation */

int reinit_controller( SIM *s )
{
  int i;

  s->controller_state = 0;
  s->cstate_start_time = s->time;
  s->cstate_elapsed_time = 0.0;

  for( i = XX; i <= ZZ; i++ )
    {
      s->root_position_d[i] = 0;
      s->root_velocity_d[i] = 0;
      s->root_w_body_d[i] = 0;
    }

  for( i = 0; i < QQ; i++ )
    {
      s->root_q_d[i] = 0;
    }
  s->root_q_d[Q0] = 1;

  for( i = 0; i < N_JOINTS; i++ )
    {
      s->joints_d[i] = 0;
      s->jointsd_d[i] = 0;
      s->trajectory_mode[i] = NO_TRAJECTORY;
    }

  for( i = 0; i < N_JOINTS; i++ )
    {
      s->controls[i] = 0;
      s->controls_ff[i] = 0;
      s->servo_mode[i] = PID_MODE;
    }

  for( i = 0; i < N_JOINTS; i++ )
    {
      s->control_inputs[i] = 0;
      s->control_inputsd[i] = 0;
      s->control_inputs_int[i] = 0;
    }

  for( i = XX; i <= ZZ; i++ )
    s->torso_perturbation_force[i] = 0;

  //init_trajectory( desired_joint_trajectory, N_JOINTS );
  //init_trajectory( current_joint_desireds, N_JOINTS );

  ///* add desired position as knot to current desireds */
  //for ( i = 0; i < N_JOINTS; i++ )
  //  add_knot_point( current_joint_desireds, i, QUINTIC_SPLINE, 0.0, 
		//    s->joints_d[i], s->jointsd_d[i], 0.0 );

  set_double_support_stance_gains( s );

  return 0;
}

/*****************************************************************************/
/* call this many times to restart a simulation */

int reinit_sim( SIM *s )
{

  srand( s->rand_seed );

  s->time = 0.0;

  init_state_two_feet_on_ground( s );

  reinit_controller( s );


  return 0;
}

/*****************************************************************************/
/* Call this once to do one time operations like memory allocation */

int init_sim( SIM *s )
{
  init_dynamics( s );
  reinit_sim( s );
  return 0;
}

/*****************************************************************************/
/*****************************************************************************/
/* Call this once to do one time operations like memory allocation */

int init_controller( SIM *s )
{
	int i;
  // no one time operations for the current controller.
  init_sim( s );
 // double x0[10]={0.0};

	//s->foot_d[LEFT][0] = s->foot[LEFT][0];
	//s->foot_d[LEFT][1] = s->foot[LEFT][1];
	//s->foot_d[LEFT][2] = s->foot[LEFT][2];

	//s->foot_d[RIGHT][0] = s->foot[RIGHT][0];
	//s->foot_d[RIGHT][1] = s->foot[RIGHT][1];
	//s->foot_d[RIGHT][2] = s->foot[RIGHT][2];

	//s->footd_d[LEFT][0] = 0.0;
	//s->footd_d[LEFT][1] = 0.0;
	//s->footd_d[LEFT][2] = 0.0;

	//s->footd_d[RIGHT][0] = 0.0;
	//s->footd_d[RIGHT][1] = 0.0;
	//s->footd_d[RIGHT][2] = 0.0;

	//s->footdd_d[LEFT][0] = 0.0;
	//s->footdd_d[LEFT][1] = 0.0;
	//s->footdd_d[LEFT][2] = 0.0;

	//s->footdd_d[RIGHT][0] = 0.0;
	//s->footdd_d[RIGHT][1] = 0.0;
	//s->footdd_d[RIGHT][2] = 0.0;

	//s->com_d[0] = s->com[0];
	//s->com_d[1] = s->com[1];
	//s->com_d[2] = s->com[2];

	//s->comd_d[0] = 0.0;
	//s->comd_d[1] = 0.0;
	//s->comd_d[2] = 0.0;

	//// copy positions
	//s->des_state[0] = s->sdfast_state[0];
	//s->des_state[1] = s->sdfast_state[1];
	//s->des_state[2] = s->sdfast_state[2];
	//s->des_state[3] = s->sdfast_state[3];
	//s->des_state[4] = s->sdfast_state[4];
	//s->des_state[5] = s->sdfast_state[5];
	//for(i=0;i<N_JOINTS;i++)	s->des_state[6+i] = s->joints_d[i];
	//s->des_state[N_U] = s->sdfast_state[N_U-4];

	//// copy velocities
	//for(i=0;i<N_U;i++)	s->des_state[N_Q+i] = 0;

	//s->contact_state=CONTACT_DSP;

  return 0;
}

/*****************************************************************************/
/*****************************************************************************/

void
run_trajectory( SIM *s, double time )
{
  int i;
  double position, velocity, acceleration;

  for ( i = 0; i < N_JOINTS; i++ )
    {
      if ( s->trajectory_mode[i] != USE_TRAJECTORY )
	continue;
      if ( !lookup_spline3( desired_joint_trajectory, i, time,
			    current_joint_desireds, &position,
			    &velocity, &acceleration ) )
	{
	  printf( "Lookup failure.\n" );
	  exit( -1 );
	}
      /* update current desired position */
      set_first_knot_to( current_joint_desireds, i, QUINTIC_SPLINE, time,
			 position, velocity, acceleration );
      s->joints_d[i] = position;
      s->jointsd_d[i] = velocity;
    }
}

/*****************************************************************************/

void
run_servos( SIM *s )
{
  int i, j;

  for( i = 0; i < N_JOINTS; i++ )
    s->control_inputs[i] = s->joints[i] - s->joints_d[i];

  for( i = 0; i < N_JOINTS; i++ )
    s->control_inputsd[i] = s->jointsd[i] - s->jointsd_d[i];

  // integrators
  for( i = 0; i < N_JOINTS; i++ )
    s->control_inputs_int[i] += s->control_inputs[i];

  // PID with feedforward
  for( i = 0; i < N_JOINTS; i++ )
    {
      if ( s->servo_mode[i] == TORQUE_MODE )
	continue;
      s->controls[i] = s->controls_ff[i];
      for( j = 0; j < N_JOINTS; j++ )
	{
	  s->controls[i] += - s->pos_gains[i][j]*s->control_inputs[j]
	    - s->vel_gains[i][j]*s->control_inputsd[j]
	    - s->int_gains[i][j]*s->control_inputs_int[j];
	}
    }
}


/*****************************************************************************/

int limit_controls( SIM *s )
{

  // Left Roll ZMP limits
  if ( s->controls[L_AAA] > s->foot_force[LEFT][ZZ]*0.05 )
    s->controls[L_AAA] = s->foot_force[LEFT][ZZ]*0.05;
  else if ( s->controls[L_AAA] < -s->foot_force[LEFT][ZZ]*0.05 )
    s->controls[L_AAA] = -s->foot_force[LEFT][ZZ]*0.05;

  // Right Roll ZMP limits
  if ( s->controls[R_AAA] > s->foot_force[RIGHT][ZZ]*0.05 )
    s->controls[R_AAA] = s->foot_force[RIGHT][ZZ]*0.05;
  else if ( s->controls[R_AAA] < -s->foot_force[RIGHT][ZZ]*0.05 )
    s->controls[R_AAA] = -s->foot_force[RIGHT][ZZ]*0.05;

  // Left Pitch ZMP limits
  if ( s->controls[L_AFE] > s->foot_force[LEFT][ZZ]*0.1 )
    s->controls[L_AFE] = s->foot_force[LEFT][ZZ]*0.1;
  else if ( s->controls[L_AFE] < -s->foot_force[LEFT][ZZ]*0.1 )
    s->controls[L_AFE] = -s->foot_force[LEFT][ZZ]*0.1;

  // Right Pitch ZMP limits
  if ( s->controls[R_AFE] > s->foot_force[RIGHT][ZZ]*0.1 )
    s->controls[R_AFE] = s->foot_force[RIGHT][ZZ]*0.1;
  else if ( s->controls[R_AFE] < -s->foot_force[RIGHT][ZZ]*0.1 )
    s->controls[R_AFE] = -s->foot_force[RIGHT][ZZ]*0.1;

  /* Bad idea, need to be able to apply torques when in air or almost
     off ground.
  // Rough Left Friction Cone Torque Limits
  if ( s->controls[L_AR] > s->foot_force[LEFT][ZZ]*0.075 )
    s->controls[L_AR] = s->foot_force[LEFT][ZZ]*0.075;
  else if ( s->controls[L_AR] < -s->foot_force[LEFT][ZZ]*0.075 )
    s->controls[L_AR] = -s->foot_force[LEFT][ZZ]*0.075;

  // Rough Right Friction Cone Torque Limits
  if ( s->controls[R_AR] > s->foot_force[RIGHT][ZZ]*0.075 )
    s->controls[R_AR] = s->foot_force[RIGHT][ZZ]*0.075;
  else if ( s->controls[R_AR] < -s->foot_force[RIGHT][ZZ]*0.075 )
    s->controls[R_AR] = -s->foot_force[RIGHT][ZZ]*0.075;
  */

  // Joint torque limits
  if ( s->controls[L_HAA] > s->torque_max_haa )
    s->controls[L_HAA] = s->torque_max_haa;
  else if ( s->controls[L_HAA] < -s->torque_max_haa )
    s->controls[L_HAA] = -s->torque_max_haa;

  // Joint torque limits
  if ( s->controls[L_HFE] > s->torque_max_hfe )
    s->controls[L_HFE] = s->torque_max_hfe;
  else if ( s->controls[L_HFE] < -s->torque_max_hfe )
    s->controls[L_HFE] = -s->torque_max_hfe;

  // Joint torque limits
  if ( s->controls[L_KFE] > s->torque_max_kfe )
    s->controls[L_KFE] = s->torque_max_kfe;
  else if ( s->controls[L_KFE] < -s->torque_max_kfe )
    s->controls[L_KFE] = -s->torque_max_kfe;

  // Joint torque limits
  if ( s->controls[L_AR] > s->torque_max_ar )
    s->controls[L_AR] = s->torque_max_ar;
  else if ( s->controls[L_AR] < -s->torque_max_ar )
    s->controls[L_AR] = -s->torque_max_ar;

  // Joint torque limits
  if ( s->controls[L_AFE] > s->torque_max_afe )
    s->controls[L_AFE] = s->torque_max_afe;
  else if ( s->controls[L_AFE] < -s->torque_max_afe )
    s->controls[L_AFE] = -s->torque_max_afe;

  // Joint torque limits
  if ( s->controls[L_AAA] > s->torque_max_aaa )
    s->controls[L_AAA] = s->torque_max_aaa;
  else if ( s->controls[L_AAA] < -s->torque_max_aaa )
    s->controls[L_AAA] = -s->torque_max_aaa;

  // Joint torque limits
  if ( s->controls[R_HAA] > s->torque_max_haa )
    s->controls[R_HAA] = s->torque_max_haa;
  else if ( s->controls[R_HAA] < -s->torque_max_haa )
    s->controls[R_HAA] = -s->torque_max_haa;

  // Joint torque limits
  if ( s->controls[R_HFE] > s->torque_max_hfe )
    s->controls[R_HFE] = s->torque_max_hfe;
  else if ( s->controls[R_HFE] < -s->torque_max_hfe )
    s->controls[R_HFE] = -s->torque_max_hfe;

  // Joint torque limits
  if ( s->controls[R_KFE] > s->torque_max_kfe )
    s->controls[R_KFE] = s->torque_max_kfe;
  else if ( s->controls[R_KFE] < -s->torque_max_kfe )
    s->controls[R_KFE] = -s->torque_max_kfe;

  // Joint torque limits
  if ( s->controls[R_AR] > s->torque_max_ar )
    s->controls[R_AR] = s->torque_max_ar;
  else if ( s->controls[R_AR] < -s->torque_max_ar )
    s->controls[R_AR] = -s->torque_max_ar;

  // Joint torque limits
  if ( s->controls[R_AFE] > s->torque_max_afe )
    s->controls[R_AFE] = s->torque_max_afe;
  else if ( s->controls[R_AFE] < -s->torque_max_afe )
    s->controls[R_AFE] = -s->torque_max_afe;

  // Joint torque limits
  if ( s->controls[R_AAA] > s->torque_max_aaa )
    s->controls[R_AAA] = s->torque_max_aaa;
  else if ( s->controls[R_AAA] < -s->torque_max_aaa )
    s->controls[R_AAA] = -s->torque_max_aaa;

  return 0;
}




//// state must be set prior and saved in SDModel *m
//void simple_inverse_dynamics(SIM *s, double *Mdes, double *torques){
//	//Primus *p = new Primus(m);
//	double tau[18];
//	double PL[6],PR[6],JL[18*6],JR[18*6];	
//	double PH[6],JH[18*6];
//	double zero[3]={0.0};
//	int i,j;
//	static int count=0;
//
//	primus_feet(m,PL,JL,PR,JR);
//
//	body_position(m,NULL,0,zero,PH,JH);
//
//	m->sdequivht(tau);
//
//	for(i=0;i<12;i++)	{
//		torques[i] = -tau[6+i];
//		if (s->foot_force_true[0][2]>0.1){
//			for(j=0;j<6;j++)	torques[i] -= JL[6*6 + 6*i + j]*s->foot_force_des[0][j];
//		}
//		if (s->foot_force_true[1][2]>0.1){
//			for(j=0;j<6;j++)	torques[i] -= JR[6*6 + 6*i + j]*s->foot_force_des[1][j];
//		}
//
//		// posture control doesn't work in quasi-static case
//		//for(j=0;j<3;j++)	torques[i] -= JH[6*6 + 6*i + 3+j]*Mdes[j];
//
//	}
//
//	//if(count%100==0){
//	//	for(i=0;i<6;i++)	printf("% f ",torques[i]);
//	//	printf("\n");
//	//	for(i=0;i<6;i++)	printf("% f ",torques[6+i]);
//	//	printf("\n\n");
//	//}
//
//	count++; 
//}

#ifdef USE_MULTITHREADING_ID
typedef struct {
	double *state;
	int m,n;
	double *A;
	double *QR;
	double *tau;
	HANDLE hThread;
	HANDLE hMutex;
	DWORD  lpThreadID;
}inversedyn_t ;

inversedyn_t inversedynData;


static void inversedynThread( void *pThis ){
	// this is the part that can be run in parallel
	// input: pointer to inversedyn structure
	inversedyn_t *share = (inversedyn_t*)pThis;

	double *QR= share->QR;
	double *A = share->A;
	double *tau = share->tau;
	
	int m = share->m;
	int n = share->n;

	int i,j;
	static int count = 0;
	
	gsl_matrix *QR_local = gsl_matrix_alloc(m,n);
	gsl_vector *tau_local = gsl_vector_alloc(n);

	gsl_matrix_view Av = gsl_matrix_view_array(A,m,n);
	gsl_matrix_view QRv = gsl_matrix_view_array(QR,m,n);
	gsl_vector_view tauv = gsl_vector_view_array(tau,n);

	WaitForSingleObject(share->hMutex,INFINITE);
		gsl_matrix_memcpy(QR_local,&Av.matrix);
	ReleaseMutex(share->hMutex);
	
	while(1){

		//===== START OF THE COMPUTATION =====//

		// do QR decomposition
		gsl_linalg_QR_decomp(QR_local,tau_local);
		//===== END OF THE COMPUTATION =====//
		
		WaitForSingleObject(share->hMutex,INFINITE);

			// copy output
			gsl_matrix_memcpy(&QRv.matrix,QR_local);
			gsl_vector_memcpy(&tauv.vector,tau_local);

			// get next input
			gsl_matrix_memcpy(QR_local,&Av.matrix);	

		ReleaseMutex(share->hMutex);

		count++;

	}

}
#endif




//void run_inverse_kinematics(SIM *s){
//	double dq[N_U]={0.0};
//	double base_q[4];
//	double base_qd[4];
//	double temp[6];
//	double zero[3]={0.0};
//	int i;
//	static int count=0;
//	double virt_pLd[6]={0.0},virt_pRd[6]={0.0},virt_comd[6]={0.0},virt_Td[3];
//	double virt_T[3];
//	static double com_int[3]={0};
//
//	set_state(m2,s->des_state);
//
//	primus_feet(m2, s->virt_foot[LEFT], NULL, s->virt_foot[RIGHT], NULL);
//
//	com_state(m2,s->virt_com,NULL);	
//	body_position(m2, NULL, 0, zero, temp, NULL);	
//	sdvcopy(&temp[3],virt_T);
//
//	for(i=0;i<3;i++){
//		//if (contact_state==RSP)
//		//	virt_pLd[i] = s->footd_d[LEFT][i] + 0.5*(s->foot_d[LEFT][i] - s->virt_foot[LEFT][i]);
//		////else
//		////	virt_pLd[i] = 0.0;
//		//if (contact_state==LSP)
//		//	virt_pRd[i] = s->footd_d[RIGHT][i] + 0.5*(s->foot_d[RIGHT][i] - s->virt_foot[RIGHT][i]);
//		////else
//		////	virt_pRd[i] = 0.0;
//
//		virt_pRd[i] = s->footd_d[RIGHT][i] + 0.3*(s->foot_d[RIGHT][i] - s->virt_foot[RIGHT][i]);
//		virt_pLd[3+i] = 0.0 + 0.3*(s->foot_d[RIGHT][3+i] - s->virt_foot[RIGHT][3+i]);
//
//		virt_pLd[i] = s->footd_d[LEFT][i] + 0.5*(s->foot_d[LEFT][i] - s->virt_foot[LEFT][i]);
//		virt_pLd[3+i] = 0.0 + 0.3*(s->foot_d[LEFT][3+i] - s->virt_foot[LEFT][3+i]);
//
//		com_int[i] += 0.001*(s->comd_d[i] - s->com[i]);
//		virt_comd[i] = 0*s->comd_d[i] + 1.5*(s->com_d[i] - s->virt_com[i]);
//		virt_Td[i] = 0.1*(0 - virt_T[i]);
//
//		// bring the virtual back to reality?
//		//virt_pLd[i] += 0.01*(s->foot[LEFT][i] - s->virt_foot[LEFT][i]);
//		//virt_pRd[i] += 0.01*(s->foot[RIGHT][i] - s->virt_foot[RIGHT][i]);
//		//virt_comd[i] += 0.01*(s->com[i] - s->virt_com[i]);
//	}
//
//
//	primus_com_ik(m2, s->des_state, virt_pLd, virt_pRd, virt_comd, virt_Td, dq);
//
//	if (0 && count%100==0){
//		for(i=0;i<6;i++)	printf("% f ",dq[6+i]);
//		printf("\n");
//		for(i=0;i<6;i++)	printf("% f ",dq[12+i]);
//		printf("\n");
//		for(i=0;i<3;i++)	printf("% f ",s->virt_com[i]);
//		printf("\n");
//		for(i=0;i<3;i++)	printf("% f ",s->virt_foot[LEFT][i]);
//		printf("\n");
//		for(i=0;i<3;i++)	printf("% f ",s->virt_foot[RIGHT][i]);
//		printf("\n\n");
//	}
//
//	//if (count>0){
//
//		base_q[0] = s->des_state[3];
//		base_q[1] = s->des_state[4];
//		base_q[2] = s->des_state[5];
//		base_q[3] = s->des_state[N_U];
//
//		s->des_state[0] += 0.001*dq[0];
//		s->des_state[1] += 0.001*dq[1];
//		s->des_state[2] += 0.001*dq[2];
//		for(i=0;i<N_JOINTS;i++)	s->des_state[6+i] += 0.001*dq[6+i];
//		for(i=0;i<N_U;i++)	s->des_state[N_Q+i] = dq[i];
//
//		sdvmul(base_q[3],&dq[3],base_qd);
//		sdvcross(base_q,&dq[3],temp);
//		sdvsub(base_qd,temp,base_qd);
//		sdvmul(0.5,base_qd,base_qd);
//		base_qd[3]= -0.5*sdvdot(base_q,&dq[3]);
//
//		s->des_state[3] += 0.001*base_qd[0];
//		s->des_state[4] += 0.001*base_qd[1];
//		s->des_state[5] += 0.001*base_qd[2];
//		s->des_state[N_U] += 0.001*base_qd[3];
//
//
//
//	//}
//
//	count++;
//}




/*****************************************************************************/

int controller( SIM *s )
{
  int i;
  static int count = 0;

  static PRIMUS_t primusVar;
  static PRIMUS_t *p = &primusVar;

  //if(s->time==0.0)	s->com_d[2] = s->com[2];

  /* Kinematics already computed by SDFAST */
  /* Forces already computed by SDFAST */

  ///* Perturbation */
  //if ( 0.8 <= s->time && s->time < 1.0 ){
  //  s->torso_perturbation_force[XX] = 150.0;
  //  s->torso_perturbation_force[YY] = 0.0;
  //}else if ( 1.8 <= s->time && s->time < 2.0 ){
  //  s->torso_perturbation_force[XX] = 0;
  //  s->torso_perturbation_force[YY] = -100.0;
  //}else{
  //  s->torso_perturbation_force[XX] = 0.0;
  //  s->torso_perturbation_force[YY] = 0.0;
  //}  
  
  if (s->time==0.0){
	my_init_controller(p);
  }

  //=====================================================
  // populate primusVar

  sdvcopy(s->com,p->com);
  sdvcopy(s->comd,p->comd);

  sdvcopy(s->cop,p->cop);

  sdvcopy(s->foot[0],p->foot[0]);
  sdvcopy(&s->foot[0][3],&p->foot[0][3]);
  sdvcopy(s->foot[1],p->foot[1]);
  sdvcopy(&s->foot[1][3],&p->foot[1][3]);

  sdvcopy(s->footd[0],p->footd[0]);
  sdvcopy(&s->footd[0][3],&p->footd[0][3]);
  sdvcopy(s->footd[1],p->footd[1]);
  sdvcopy(&s->footd[1][3],&p->footd[1][3]);

  p->time = s->time;

  sdvcopy(s->foot_force[0],p->foot_force[0]);
  sdvcopy(s->foot_force[1],p->foot_force[1]);

  // copy positions
  p->state[0] = s->sdfast_state[0];
  p->state[1] = s->sdfast_state[1];
  p->state[2] = s->sdfast_state[2];
  p->state[3] = s->sdfast_state[3];
  p->state[4] = s->sdfast_state[4];
  p->state[5] = s->sdfast_state[5];
  for(i=0;i<N_J;i++)	p->state[6+i] = s->joints_true[i];
  p->state[N_U] = s->sdfast_state[N_U-4];

  // copy velocities
  p->state[N_Q+0] = s->sdfast_state[N_Q-4+0];
  p->state[N_Q+1] = s->sdfast_state[N_Q-4+1];
  p->state[N_Q+2] = s->sdfast_state[N_Q-4+2];
  p->state[N_Q+3] = s->sdfast_state[N_Q-4+3];
  p->state[N_Q+4] = s->sdfast_state[N_Q-4+4];
  p->state[N_Q+5] = s->sdfast_state[N_Q-4+5];
  for(i=0;i<N_J;i++)	p->state[N_Q+6+i] = s->jointsd_true[i];

  sdvcopy(s->root_euler_true,p->torso);
  sdvcopy(s->root_w_body_true,p->torsod);

  //=====================================================
  //======================================================

  my_controller(p); 

  // copy info from PRIMUS_t struct to SIM struct
  for(i=0;i<N_J;i++)	s->controls_ff[i] = p->torque_ff[i];
  sdvcopy(p->com_d,s->com_d);
  sdvcopy(p->comd_d,s->comd_d);
  sdvcopy(p->comdd_d,s->comdd_d);
  sdvcopy(p->cop_d,s->cop_d);
  sdvcopy(&p->foot_d[LEFT][0],&s->foot_d[LEFT][0]);
  sdvcopy(&p->foot_d[LEFT][3],&s->foot_d[LEFT][3]);
  sdvcopy(&p->foot_d[RIGHT][0],&s->foot_d[RIGHT][0]);
  sdvcopy(&p->foot_d[RIGHT][3],&s->foot_d[RIGHT][3]);
  sdvcopy(&p->footd_d[LEFT][0],&s->footd_d[LEFT][0]);
  sdvcopy(&p->footd_d[LEFT][3],&s->footd_d[LEFT][3]);
  sdvcopy(&p->footd_d[RIGHT][0],&s->footd_d[RIGHT][0]);
  sdvcopy(&p->footd_d[RIGHT][3],&s->footd_d[RIGHT][3]);
  sdvcopy(&p->footdd_d[LEFT][0],&s->footdd_d[LEFT][0]);
  sdvcopy(&p->footdd_d[LEFT][3],&s->footdd_d[LEFT][3]);
  sdvcopy(&p->footdd_d[RIGHT][0],&s->footdd_d[RIGHT][0]);
  sdvcopy(&p->footdd_d[RIGHT][3],&s->footdd_d[RIGHT][3]);
  s->controller_state=p->controller_state;
  s->contact_state=p->contact_state;
  s->step_des[0] = p->step_des[0];
  s->step_des[1] = p->step_des[1];
  s->cstate_elapsed_time = p->cstate_elapsed_time;
  sdvcopy(p->com_est,s->com_est);
  sdvcopy(p->comd_est,s->comd_est);
  s->num_active_threads = p->num_active_threads;
  s->preview_step_time = p->preview_step_time;
  s->preview_cost = p->preview_cost;
  s->preview_comp_time = p->preview_comp_time;

  sdvcopy(p->com_force_est,s->com_force_est);

  //for(i=0;i<N_JOINTS;i++)	s->joints_d[i] = p->state_des[6+i];
  //for(i=0;i<N_JOINTS;i++)	s->jointsd_d[i] = p->state_des[N_Q+6+i];
  for(i=0;i<N_Q+N_U;i++)	s->des_state[i] = p->state_des[i];

  run_servos( s );

 // if (count%100==0){
	//for(i=0;i<m->NU();i++)	printf("% f ",dq[i]);
	//printf("\n\n");

	////for(i=0;i<8;i++)	printf("% f ",libm_state[i]);
	//printf("x = % f ",libm_state[1]-0.5*(libm_state[5]+libm_state[7]));
	//printf("wL = % f ",libm.wL);
	//printf("\n\n");
	//
 // }

  limit_controls( s );

  count++;

  return 0;
}

/*****************************************************************************/

}

