/*****************************************************************************/
/*
  data.c: save data from simulation
*/
/*****************************************************************************/
/*
th
th_d
thd
u
ufb
uff
*/
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../useful/mrdplot/mrdplot.h"

#include "main.h"
#include "main2.h"

/*****************************************************************************/

extern SIM sim;

extern char joint_names[][20];

/*****************************************************************************/

#define MAX_N_POINTS 100000
#define MAX_CHANNELS 500
#define  MAX_CHARS 20

int n_points = 0;
float data[MAX_N_POINTS*MAX_CHANNELS];

typedef struct{
	char units[MAX_CHARS];
	char name[MAX_CHARS];
	double *ptr;
} DataPoint_t;

DataPoint_t datapoints[MAX_CHANNELS];
static int n_channels = 0;



void add_datapoint(char *name, char *units, double *ptr){
	if (n_channels<MAX_CHANNELS){
		DataPoint_t *dptr = &datapoints[n_channels];

		  strncpy(dptr->name,name,MAX_CHARS-1);
		  dptr->name[MAX_CHARS-1] = '\0';
		  strncpy(dptr->units,units,MAX_CHARS-1);
		  dptr->units[MAX_CHARS-1] = '\0';

		datapoints[n_channels].ptr = ptr;
		n_channels++;
	}
}

void init_data( SIM *s )
{
  int i;
  char buff[MAX_CHARS];

  /*
  printf( "N_CHANNELS: %d; M_L_TORSO_BOTTOM: %d; last: %d\n", N_CHANNELS,
	  M_L_TORSO_BOTTOM, M_R_ANKLE_COMMAND );
  */

  //for( i = 0; i < N_CHANNELS; i++ )
  //  {
  //    names[i] = "xxx";
  //    units[i] = "xxx";
  //  }

  add_datapoint("time", "s", &s->time);

  add_datapoint("x", "m", &s->root_position[0]);
  add_datapoint("y", "m", &s->root_position[1]);
  add_datapoint("z", "m", &s->root_position[2]);

  add_datapoint("a_q0", "rad", &s->root_q[1]);
  add_datapoint("a_q1", "rad", &s->root_q[2]);
  add_datapoint("a_q2", "rad", &s->root_q[3]);
  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"%s_th",joint_names[i]);
	  add_datapoint(buff, "rad", &s->joints_true[i]);
  }
  add_datapoint("a_q3", "rad", &s->root_q[0]);

  add_datapoint("xd", "m/s", &s->root_velocity[0]);
  add_datapoint("yd", "m/s", &s->root_velocity[1]);
  add_datapoint("zd", "m/s", &s->root_velocity[2]);

  add_datapoint("a_wx", "rad/s", &s->root_w_body[0]);
  add_datapoint("a_wy", "rad/s", &s->root_w_body[1]);
  add_datapoint("a_wz", "rad/s", &s->root_w_body[2]);

  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"%s_thd",joint_names[i]);
	  add_datapoint(buff, "rad/s", &s->jointsd_true[i]);
  }
  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"%s_th_d",joint_names[i]);
	  add_datapoint(buff, "rad", &s->joints_d[i]);
  }
  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"%s_thd_d",joint_names[i]);
	  add_datapoint(buff, "rad/s", &s->jointsd_d[i]);
  }
  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"%s_trq",joint_names[i]);
	  add_datapoint(buff, "Nm", &s->controls[i]);
  }

  add_datapoint("com_x", "m", &s->com[0]);
  add_datapoint("com_y", "m", &s->com[1]);
  add_datapoint("com_z", "m", &s->com[2]);

  add_datapoint("com_x_d", "m", &s->com_d[0]);
  add_datapoint("com_y_d", "m", &s->com_d[1]);
  add_datapoint("com_z_d", "m", &s->com_d[2]);

  add_datapoint("comd_x", "m/s", &s->comd[0]);
  add_datapoint("comd_y", "m/s", &s->comd[1]);
  add_datapoint("comd_z", "m/s", &s->comd[2]);

  add_datapoint("comd_x_d", "m/s", &s->comd_d[0]);
  add_datapoint("comd_y_d", "m/s", &s->comd_d[1]);
  add_datapoint("comd_z_d", "m/s", &s->comd_d[2]);

  add_datapoint("comdd_x_d", "m/s", &s->comdd_d[0]);
  add_datapoint("comdd_y_d", "m/s", &s->comdd_d[1]);
  add_datapoint("comdd_z_d", "m/s", &s->comdd_d[2]);


  
  add_datapoint("l_foot_x", "m", &s->foot[LEFT][0]);	
  add_datapoint("l_foot_y", "m", &s->foot[LEFT][1]);	
  add_datapoint("l_foot_z", "m", &s->foot[LEFT][2]);

  add_datapoint("r_foot_x", "m", &s->foot[RIGHT][0]);	
  add_datapoint("r_foot_y", "m", &s->foot[RIGHT][1]);	
  add_datapoint("r_foot_z", "m", &s->foot[RIGHT][2]);

  add_datapoint("l_foot_x_d", "m", &s->foot_d[LEFT][0]);	
  add_datapoint("l_foot_y_d", "m", &s->foot_d[LEFT][1]);	
  add_datapoint("l_foot_z_d", "m", &s->foot_d[LEFT][2]);

  add_datapoint("r_foot_x_d", "m", &s->foot_d[RIGHT][0]);	
  add_datapoint("r_foot_y_d", "m", &s->foot_d[RIGHT][1]);	
  add_datapoint("r_foot_z_d", "m", &s->foot_d[RIGHT][2]);
//
  add_datapoint("l_footdd_x_d", "m", &s->footdd_d[LEFT][0]);	
  add_datapoint("l_footdd_y_d", "m", &s->footdd_d[LEFT][1]);	
  add_datapoint("l_footdd_z_d", "m", &s->footdd_d[LEFT][2]);

  add_datapoint("r_footdd_x_d", "m", &s->footdd_d[RIGHT][0]);	
  add_datapoint("r_footdd_y_d", "m", &s->footdd_d[RIGHT][1]);	
  add_datapoint("r_footdd_z_d", "m", &s->footdd_d[RIGHT][2]);
//  add_datapoint("l_foot_xd", "m/s", &s->footd[LEFT][0]);	
//  add_datapoint("l_foot_yd", "m/s", &s->footd[LEFT][1]);	
//  add_datapoint("l_foot_zd", "m/s", &s->footd[LEFT][2]);
//
//  add_datapoint("r_foot_xd", "m/s", &s->footd[RIGHT][0]);	
//  add_datapoint("r_foot_yd", "m/s", &s->footd[RIGHT][1]);	
//  add_datapoint("r_foot_zd", "m/s", &s->footd[RIGHT][2]);
//
//  add_datapoint("l_foot_xd_d", "m/s", &s->footd_d[LEFT][0]);	
//  add_datapoint("l_foot_yd_d", "m/s", &s->footd_d[LEFT][1]);	
//  add_datapoint("l_foot_zd_d", "m/s", &s->footd_d[LEFT][2]);
//
//  add_datapoint("r_foot_xd_d", "m/s", &s->footd_d[RIGHT][0]);	
//  add_datapoint("r_foot_yd_d", "m/s", &s->footd_d[RIGHT][1]);	
//  add_datapoint("r_foot_zd_d", "m/s", &s->footd_d[RIGHT][2]);
//
  add_datapoint("l_foot_fx", "N", &s->foot_force[LEFT][0]);	
  add_datapoint("l_foot_fy", "N", &s->foot_force[LEFT][1]);	
  add_datapoint("l_foot_fz", "N", &s->foot_force[LEFT][2]);
  add_datapoint("l_foot_mx", "Nm", &s->foot_force[LEFT][3]);	
  add_datapoint("l_foot_my", "Nm", &s->foot_force[LEFT][4]);	
  add_datapoint("l_foot_mz", "Nm", &s->foot_force[LEFT][5]);

  add_datapoint("r_foot_fx", "N", &s->foot_force[RIGHT][0]);	
  add_datapoint("r_foot_fy", "N", &s->foot_force[RIGHT][1]);	
  add_datapoint("r_foot_fz", "N", &s->foot_force[RIGHT][2]);
  add_datapoint("r_foot_mx", "Nm", &s->foot_force[RIGHT][3]);	
  add_datapoint("r_foot_my", "Nm", &s->foot_force[RIGHT][4]);	
  add_datapoint("r_foot_mz", "Nm", &s->foot_force[RIGHT][5]);

  add_datapoint("l_foot_fx_d", "N", &s->foot_force_des[LEFT][0]);	
  add_datapoint("l_foot_fy_d", "N", &s->foot_force_des[LEFT][1]);	
  add_datapoint("l_foot_fz_d", "N", &s->foot_force_des[LEFT][2]);
  add_datapoint("l_foot_mx_d", "Nm", &s->foot_force_des[LEFT][3]);	
  add_datapoint("l_foot_my_d", "Nm", &s->foot_force_des[LEFT][4]);	
  add_datapoint("l_foot_mz_d", "Nm", &s->foot_force_des[LEFT][5]);

  add_datapoint("r_foot_fx_d", "N", &s->foot_force_des[RIGHT][0]);	
  add_datapoint("r_foot_fy_d", "N", &s->foot_force_des[RIGHT][1]);	
  add_datapoint("r_foot_fz_d", "N", &s->foot_force_des[RIGHT][2]);
  add_datapoint("r_foot_mx_d", "Nm", &s->foot_force_des[RIGHT][3]);	
  add_datapoint("r_foot_my_d", "Nm", &s->foot_force_des[RIGHT][4]);	
  add_datapoint("r_foot_mz_d", "Nm", &s->foot_force_des[RIGHT][5]);
//
//  add_datapoint("l_foot_status", "-", &s->foot_status[LEFT]);
//  add_datapoint("r_foot_status", "-", &s->foot_status[RIGHT]);
//
//  add_datapoint("torso_ex", "rad", &s->root_euler_true[XX]);
//  add_datapoint("torso_ey", "rad", &s->root_euler_true[YY]);
//  add_datapoint("torso_ez", "rad", &s->root_euler_true[ZZ]);
//

  add_datapoint("virt_x", "m", &s->des_state[0]);
  add_datapoint("virt_y", "m", &s->des_state[1]);
  add_datapoint("virt_z", "m", &s->des_state[2]);
  add_datapoint("virt_q0", "rad", &s->des_state[3]);
  add_datapoint("virt_q1", "rad", &s->des_state[4]);
  add_datapoint("virt_q2", "rad", &s->des_state[5]);
  for(i=0;i<N_JOINTS;i++){
	  sprintf(buff,"virt_%s_th",joint_names[i]);
	  add_datapoint(buff, "rad", &s->des_state[6+i]);
  }
  add_datapoint("virt_q3", "rad", &s->des_state[6+N_JOINTS]);

  add_datapoint("virt_com_x", "m", &s->virt_com[0]);
  add_datapoint("virt_com_y", "m", &s->virt_com[1]);
  add_datapoint("virt_com_z", "m", &s->virt_com[2]);

  
  add_datapoint("cop_x", "m", &s->cop[0]);
  add_datapoint("cop_y", "m", &s->cop[1]);

  add_datapoint("cop_x_d", "m", &s->cop_d[0]);
  add_datapoint("cop_y_d", "m", &s->cop_d[1]);

  add_datapoint("cap_x", "m", &s->cap[0]);
  add_datapoint("cap_y", "m", &s->cap[1]);

  add_datapoint("step_des_x", "m", &s->step_des[0]);
  add_datapoint("step_des_y", "m", &s->step_des[1]);
  add_datapoint("step_des_2_x", "m", &s->step_des[2]);
  add_datapoint("step_des_2_y", "m", &s->step_des[3]);

  
  add_datapoint("com_est_x", "m", &s->com_est[0]);
  add_datapoint("com_est_y", "m", &s->com_est[1]);
  add_datapoint("comd_est_x", "m/s", &s->comd_est[0]);
  add_datapoint("comd_est_y", "m/s", &s->comd_est[1]);
  add_datapoint("cop_est_x", "m", &s->cop_est[0]);
  add_datapoint("cop_est_y", "m", &s->cop_est[1]);
  add_datapoint("com_force_est_x", "N", &s->com_force_est[0]);
  add_datapoint("com_force_est_y", "N", &s->com_force_est[1]);

  add_datapoint("push_force_x", "N", &s->torso_perturbation_force[0]);
  add_datapoint("push_force_y", "N", &s->torso_perturbation_force[1]);

  
  add_datapoint("cstate", "-", &s->cstate);

  add_datapoint("realtime", "-", &s->realtime);


  add_datapoint("num_active_threads", "-", &s->num_active_threads);
  add_datapoint("preview_cost", "-", &s->preview_cost);
  add_datapoint("preview_step_time", "-", &s->preview_step_time);
  add_datapoint("preview_comp_time", "-", &s->preview_comp_time);


//
//names[M_VIRT_COM_X] = "virt_com_x";
//names[M_VIRT_COM_Y] = "virt_com_y";
//names[M_VIRT_COM_Z] = "virt_com_z";
//
//names[M_VIRT_L_FOOT_X] = "virt_pl_x";
//names[M_VIRT_L_FOOT_Y] = "virt_pl_y";
//names[M_VIRT_L_FOOT_Z] = "virt_pl_z";
//
//names[M_VIRT_R_FOOT_X] = "virt_pr_x";
//names[M_VIRT_R_FOOT_Y] = "virt_pr_y";
//names[M_VIRT_R_FOOT_Z] = "virt_pr_z";
//
//
//	names[M_VIRT_DX] = "virt_dx";
//	names[M_VIRT_DY] = "virt_dy";
//	names[M_VIRT_DZ] = "virt_dz";
//	names[M_VIRT_W0] = "virt_w0";
//	names[M_VIRT_W1] = "virt_w1";
//	names[M_VIRT_W2] = "virt_w2";
//	names[M_VIRT_L_HAA_THD] = "virt_l_haa_thd";
//	names[M_VIRT_L_HFE_THD] = "virt_l_hfe_thd";
//	names[M_VIRT_L_KFE_THD] = "virt_l_kfe_thd";
//	names[M_VIRT_L_AR_THD] = "virt_l_ar_thd";
//	names[M_VIRT_L_AFE_THD] = "virt_l_afe_thd";
//	names[M_VIRT_L_AAA_THD] = "virt_l_aaa_thd";
//	names[M_VIRT_R_HAA_THD] = "virt_r_haa_thd";
//	names[M_VIRT_R_HFE_THD] = "virt_r_hfe_thd";
//	names[M_VIRT_R_KFE_THD] = "virt_r_kfe_thd";
//	names[M_VIRT_R_AR_THD] = "virt_r_ar_thd";
//	names[M_VIRT_R_AFE_THD] = "virt_r_afe_thd";
//	names[M_VIRT_R_AAA_THD] = "virt_r_aaa_thd";
//  

//
//  names[M_L_COP_X] = "l_cop_x";
//  names[M_L_COP_Y] = "l_cop_y";
//  names[M_L_COP_Z] = "l_cop_z";
//  
//  names[M_R_COP_X] = "r_cop_x";
//  names[M_R_COP_Y] = "r_cop_y";
//  names[M_R_COP_Z] = "r_cop_z";
//
//  


  /*
  for( i = 0; i < N_CHANNELS; i++ )
    {
      printf( "%d %s %s\n", i, names[i], units[i] );
    }
  */
}

/*****************************************************************************/

int save_data( SIM *s )
{
  static int myindex = 0;
  int i;
  double *ptr;

  if ( n_points >= MAX_N_POINTS )
    return 0;

  for(i=0;i<n_channels;i++){
	  ptr = (double*)datapoints[i].ptr;
	  data[myindex + i] = (float)*ptr;
  }
  myindex+=n_channels;
  n_points++;

  return 0;
}

/*****************************************************************************/

int write_matlab_file( char *filename )
{
  FILE *stream;
  int i;
  static int myindex = 0;

  stream = fopen( filename, "w" );
  if ( stream == NULL )
    {
      fprintf( stderr, "Can't open %s for write.\n", filename );
      exit( -1 );
    }
  for ( i = 0; i < n_points; i++ )
    {
      fprintf( stream, "%g ", data[myindex] );
      fprintf( stream, "\n" );
      myindex += n_channels;
    }
  fclose( stream );

  return 0;
}

/*****************************************************************************/

int write_the_mrdplot_file( SIM *s )
{
  MRDPLOT_DATA *d; 
  int i;
  char *names[MAX_CHANNELS];
  char *units[MAX_CHANNELS];

  d = malloc_mrdplot_data( 0, 0 );
  d->filename = generate_file_name();
  d->n_channels = n_channels;
  d->n_points = n_points;
  d->total_n_numbers = d->n_channels*d->n_points;
  d->frequency = (float) (1/s->time_step);
  d->data = data;

  for(i=0;i<n_channels;i++){
	names[i] = datapoints[i].name;
	units[i] = datapoints[i].units;
	//sprintf(&d->units[i*MAX_CHARS],"%s ",datapoints[i].units);
  }
  d->names = names;
  d->units = units;

  write_mrdplot_file( d );

  // write_matlab_file( "dd" );

  return 0;
}

/*****************************************************************************/
