#ifndef _LIBMDYN_
#define _LIBMDYN_

#define ZERO(x)	(0.5*(x[2]+x[3]))
#define WIDTH(x) (sqrt((x[4]-x[6])*(x[4]-x[6]) + (x[5]-x[7])*(x[5]-x[7])))

#ifndef CLAMP
#define CLAMP( x,min,max )((x)<(min)?(min):(x)>(max)?(max):(x))
#endif

#define MAXTRAJ	5000
#define DSP     0
#define LSP     1
#define RSP     2
#define LSP2	3
#define RSP2    4

#define CRASH_COST	(1.0e20)  

typedef struct{
	double mass;	// mass
	double L;	// height
	double dsp_ratio;	// dsp ratio
	double dx;	// half foot length
	double dy;	// half foot width
	double Wd;	// default stance width
	double Ed;	// desired energy
	double dt;
	double KE;
	double a,b;
    double theta; // angle of rotated frame
    double x2;
    double dx2;
    double y2;
    double dy2;
    double x0;
    double y0;
    double W;
    double D;
    double Wx;  // x distance between feet
    double Wy;  // y distance between feet
    double wL;
    double wR;  
	double dwL;
	double dwR;
    double xtraj[MAXTRAJ][8],utraj[MAXTRAJ][2],ftraj[MAXTRAJ][12],straj[MAXTRAJ][1];
    int traj_idx;
    double energy_cost;
    double position_cost;
    double stance_cost;
    double step_cost;
} LiBMDyn_t;

void find_step2_state(double *x0, double *ps, double L, double dx, double dy, double W, double t, double *x);
void find_step2(double *x0, double *ps, double L, double dx, double dy, double W, double *pc, double *tc, double *xc);

void find_step(double *x0, double *ps, double L, double dx, double dy, double W, double *pc, double *x1, double *x2, double *t1, double *t2);
void find_step_state(double *x0, double *ps, double L, double dx, double dy, double W, double *pc, double time, double *x);

void forward_dyn(LiBMDyn_t *m, double *x0, double *u, double *x1, double *F);
void inverse_dyn(LiBMDyn_t *m, double *x0, double *ddx, double *Mdes, double *u, double *F);
void energy_control(LiBMDyn_t *m, double *x0, double *u);
void update_weights(LiBMDyn_t *m, double *x);
double simulate_onestep(LiBMDyn_t *m, double *x0, double *step_dist, int debug);
double lookahead(LiBMDyn_t *m, int N, double *step_set, int M, double *x0, double *best_action);

#endif  