#ifndef PREVIEW2_FORCE_H
#define PREVIEW2_FORCE_H



#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/io.hpp>

using namespace boost::numeric::ublas;

//#define MAX_N	25
#define USE_PRECOMPUTED_QP

#ifdef USE_PRECOMPUTED_QP
#define MAX_TIME_DIVISIONS	20

typedef struct{
	matrix<double>P;
	matrix<double>Z;
	matrix<double>Q;
	double Qtrace;
} preview2force_qp_t;
#endif

#ifdef __cplusplus
class Preview2force{
public:

	Preview2force(int N);
	double run(double *x, double L, double t, double T, int N, double Tstep, double Tdsp, double *step, double *traj, double *ttraj);

private:

	int MAX_N;

int preview_initialized;

matrix<double>P;
matrix<double>Z;
matrix<double>UN;
matrix<double>U0;
matrix<double>U1;
matrix<double>U2;
matrix<double>U3;
matrix<double>U4;

#ifdef USE_PRECOMPUTED_QP
preview2force_qp_t P2QP[MAX_TIME_DIVISIONS];
double Qtrace;
#endif

double gain_a;
double gain_b;
double gain_c;
double gain_d;
double gain_ex;
double gain_ey;
double gain_f;


void preview_init(matrix<double> *Q, matrix<double> *p, matrix<double> *Ain, matrix<double> *bin, double L, double t, double T, int N, double Tstep, double Tdsp);
void preview_update(matrix<double> *Q, matrix<double> *p, matrix<double> *Ain, matrix<double> *bin, double *x, double L, double t, double T, int N, double Tstep, double Tdsp);
void preview_simulate(double *x, double L, double t, double T, int N, double Tstep, double Tdsp, vector<double> u, double *traj, double *ttraj);

#ifdef USE_PRECOMPUTED_QP
void precompute_one(preview2force_qp_t *prob, double t, double L, double T, int N, double Tstep, double Tdsp);
void precompute(double L, double T, int N, double Tstep, double Tdsp);
#endif

};
#else
typedef struct Preview2force Preview2force;
#endif


#endif