#ifdef _USE_BOOST

#ifdef MATLAB
#include "mex.h"
#endif

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/io.hpp>

#include <math.h>
#include "uquadprog.hpp"

using namespace boost::numeric::ublas;

static double sign(double x){
	if (x>0)	return 1.0;
	else		return -1.0;
}

static double min(double a, double b){
	if (a<b)	return a;
	else		return b;
}

static double max(double a, double b){
	if (a>b)	return a;
	else		return b;
}

matrix<double> cross_matrix(matrix<double> V){
	matrix<double> M(3,3);
	M(0,0)=  0.0;   M(0,1)=-V(2,0); M(0,2)= V(1,0);
	M(1,0)= V(2,0); M(1,1)=  0.0;   M(1,2)=-V(0,0);
	M(2,0)=-V(1,0); M(2,1)= V(0,0); M(2,2)= 0.0;
	return M;
}

void preview_recurse(matrix<double> *P, matrix<double> *Z, matrix<double> A, matrix<double> B, matrix<double> C, int i, int N){
	
	matrix_range<matrix<double>>Px(*P,range(3*i,3*i+3),range(0,3+N));	
	matrix_range<matrix<double>>Pu(*P,range(3*i,3*i+3),range(3+i,3+i+1));		
	matrix_range<matrix<double>>Zi(*Z,range(i,i+1),range(0,3+N));

	if (i==0){
		matrix_range<matrix<double>>P0(*P,range(0,3),range(0,3));
		P0 = A;
	}else{
		matrix_range<matrix<double>>P0(*P,range(3*(i-1),3*(i-1)+3),range(0,3+N));	
		Px = prod(A,P0);
	}

	Pu = B;	
	Zi = prod(C,Px);

	if (i<N-1)	return preview_recurse(P,Z,A,B,C,i+1,N);

	return;
}

void matrix_vertcat(matrix<double>*A,matrix<double>B){
    A->resize(A->size1()+B.size1(),A->size2(),true);
    matrix_range<matrix<double>>A2(*A,range(A->size1()-B.size1(),A->size1()),range(0,A->size2()));
    A2 = B;
}


void preview_init(matrix<double> *Q, matrix<double> *p, matrix<double> *Ain, matrix<double> *bin, double *x, double L, double T, int N, double Tstep, double Tdsp){
	int i,j;
	matrix<double>A(3,3);
	matrix<double>B(3,1);
	matrix<double>C(1,3);
	matrix<double>P(3*N,3+N);
	matrix<double>Z(N,3+N);
	matrix<double>UN(N,1);
	matrix<double>U0(N,1);
	matrix<double>U1(N,1);
	matrix<double>U2(N,1);
	matrix<double>U3(N,1);
	matrix<double>U4(N,1);

	matrix<double>x0(3,1);
	matrix<double>y0(3,1);
	x0(0,0) = x[0];
	x0(1,0) = x[1];
	x0(2,0) = x[2];
	double xf0 = x[3];
	y0(0,0) = x[4];
	y0(1,0) = x[5];
	y0(2,0) = x[6];
	double yf0 = x[7];
	double xs0 = x[8];
	double ys0 = x[9];

	double a = 1.0e-4;
	double b = 1.0e-2;
	double c = 1.0e-1;
	double d = 1.0e-5;
	double e = 6.0e-0;

	A(0,0) = 1.0; A(0,1) = T;   A(0,2) = T*T*0.5;
	A(1,0) = 0.0; A(1,1) = 1.0; A(1,2) = T;
	A(2,0) = 0.0; A(2,1) = 0.0; A(2,2) = 1.0;

	B(0,0) = T*T*T/6.0;
	B(1,0) = T*T*0.5;
	B(2,0) = T;

	C(0,0) = 1.0;	C(0,1) = 0.0;	C(0,2) = -L/9.81;

	for(i=0;i<N;i++){
		UN(i,0) = 1.0;
		if ((i<ceil(Tdsp/T) && (Tdsp!=0.0)) || (Tdsp+Tstep)<=0.0){
			U1(i,0)=U2(i,0)=U3(i,0)=U4(i,0)=0.0;
			U0(i,0)=1.0;
		}else if (i < ceil((Tdsp+Tstep)/T)){
			U0(i,0)=U2(i,0)=U3(i,0)=U4(i,0)=0.0;
			U1(i,0)=1.0;
		}else if (i < ceil((Tdsp+Tstep+Tdsp)/T)){
			U0(i,0)=U1(i,0)=U3(i,0)=U4(i,0)=0.0;
			U2(i,0)=1.0;
		}else if (i < ceil((Tdsp+Tstep+Tdsp+Tstep)/T)){
			U0(i,0)=U1(i,0)=U2(i,0)=U4(i,0)=0.0;
			U3(i,0)=1.0;
		}else{
			U0(i,0)=U1(i,0)=U2(i,0)=U3(i,0)=0.0;
			U4(i,0)=1.0;            
        }
	}

    
//     std::cout << UN << std::endl;
//         std::cout << U0 << std::endl;
//             std::cout << U1 << std::endl;
//                 std::cout << U2 << std::endl;
//                     std::cout << U3 << std::endl;
//                         std::cout << U4 << std::endl;
    
	P.clear();
	Z.clear();

	preview_recurse(&P,&Z,A,B,C,0,N);

	// position
	matrix_slice<matrix<double>> Ppx(P,slice(0,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Ppu(P,slice(0,3,N),slice(3,1,N));
	// velocity
	matrix_slice<matrix<double>> Pvx(P,slice(1,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Pvu(P,slice(1,3,N),slice(3,1,N));
	// acceleration
	matrix_slice<matrix<double>> Pax(P,slice(2,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Pau(P,slice(2,3,N),slice(3,1,N));
	// zmp
	matrix_slice<matrix<double>> Zx(Z,slice(0,1,N),slice(0,1,3));
	matrix_slice<matrix<double>> Zu(Z,slice(0,1,N),slice(3,1,N));   
    
        
	
	matrix<double>H(N+2,N+2);
	identity_matrix<double>IN(N,N);
	identity_matrix<double>I1(1,1);

	matrix_range<matrix<double>> H00(H,range(0  ,N  ),range(0  ,N  ));
	matrix_range<matrix<double>> H01(H,range(0  ,N  ),range(N  ,N+1));
	matrix_range<matrix<double>> H02(H,range(0  ,N  ),range(N+1,N+2));
	matrix_range<matrix<double>> H10(H,range(N  ,N+1),range(0  ,N  ));
	matrix_range<matrix<double>> H11(H,range(N  ,N+1),range(N  ,N+1));    
	matrix_range<matrix<double>> H12(H,range(N  ,N+1),range(N+1,N+2));
	matrix_range<matrix<double>> H20(H,range(N+1,N+2),range(0  ,N  ));
	matrix_range<matrix<double>> H21(H,range(N+1,N+2),range(N  ,N+1));    
	matrix_range<matrix<double>> H22(H,range(N+1,N+2),range(N+1,N+2));

	H00 = + a*prod(trans(Zu),Zu) 
          + b*prod(trans(Ppu),Ppu) 
		  + c*prod(trans(Pvu),Pvu)
		  + d*IN;
	
	H01 = -a*prod(trans(Zu),(U2+U3+0.5*U4)) - 0.5*b*prod(trans(Ppu),UN);
	H10 = trans(H01);
    
    H02 = -0.5*a*prod(trans(Zu),U4)-0.5*b*prod(trans(Ppu),UN);
    H20 = trans(H02);
        
	H11 = a*prod(trans(U2+U3+0.5*U4),U2+U3+0.5*U4)+0.25*b*prod(trans(UN),UN)+2*e*I1;
    
    H12 = 0.5*a*prod(trans(U2+U3+0.5*U4),U4)+0.25*b*prod(trans(UN),UN)-e*I1;
    H21 = trans(H12);
    
    H22 = 0.25*a*prod(trans(U4),U4)+0.25*b*prod(trans(UN),UN)+e*I1;
    
    

	Q->clear();
	matrix_range<matrix<double>> Q00(*Q,range(0,N+2),range(0,N+2));
	matrix_range<matrix<double>> Q11(*Q,range(N+2,2*(N+2)),range(N+2,2*(N+2)));

	Q00 = H;
	Q11 = H;

	p->clear();
	matrix_range<matrix<double>> px1(*p,range(0,N),range(0,1));
	matrix_range<matrix<double>> px2(*p,range(N,N+1),range(0,1));
	matrix_range<matrix<double>> px3(*p,range(N+1,N+2),range(0,1));
    
	matrix_range<matrix<double>> py1(*p,range(N+2,N+2+N),range(0,1));
	matrix_range<matrix<double>> py2(*p,range(N+2+N,N+2+N+1),range(0,1));
	matrix_range<matrix<double>> py3(*p,range(N+2+N+1,N+2+N+2),range(0,1));


	px1 = trans(   -a*prod(trans(0.5*(xs0+xf0)*U0+xf0*U1-prod(Zx,x0)),Zu)                +b*prod(trans(prod(Ppx,x0)),Ppu)+c*prod(trans(prod(Pvx,x0)),Pvu));
    px2 = trans(    a*prod(trans(0.5*(xs0+xf0)*U0+xf0*U1-prod(Zx,x0)),(U2+U3+0.5*U4))-0.5*b*prod(trans(prod(Ppx,x0)),UN)-e*(xf0+0.0)*I1+e*0.0*I1         );
    px3 = trans(a*0.5*prod(trans(0.5*(xs0+xf0)*U0+xf0*U1-prod(Zx,x0)),U4)            -0.5*b*prod(trans(prod(Ppx,x0)),UN)-e*0.0*I1                        ); 
    
	py1 = trans(   -a*prod(trans(0.5*(ys0+yf0)*U0+yf0*U1-prod(Zx,y0)),Zu)                +b*prod(trans(prod(Ppx,y0)),Ppu)+c*prod(trans(prod(Pvx,y0)),Pvu));
    py2 = trans(    a*prod(trans(0.5*(ys0+yf0)*U0+yf0*U1-prod(Zx,y0)),(U2+U3+0.5*U4))-0.5*b*prod(trans(prod(Ppx,y0)),UN)-e*(yf0+sign(ys0-yf0)*0.24)*I1+e*(-sign(ys0-yf0)*0.24)*I1);
    py3 = trans(a*0.5*prod(trans(0.5*(ys0+yf0)*U0+yf0*U1-prod(Zx,y0)),U4)            -0.5*b*prod(trans(prod(Ppx,y0)),UN)-e*(-sign(ys0-yf0)*0.24)*I1       ); 

	Ain->clear();
	bin->clear();
    
    // =============ZMP constraints=============

	matrix_range<matrix<double>>Axu(*Ain, range(0,2*N),range(0  ,N));
	matrix_range<matrix<double>>Axf1(*Ain,range(0,2*N),range(N  ,N+1));
	matrix_range<matrix<double>>Axf2(*Ain,range(0,2*N),range(N+1,N+2));
	matrix_range<matrix<double>>bx(*bin,  range(0,2*N),range(0  ,1));

	matrix_range<matrix<double>>Ayu(*Ain, range(2*N,4*N),range(N+2    ,N+2+N));
	matrix_range<matrix<double>>Ayf1(*Ain,range(2*N,4*N),range(N+2+N  ,N+2+N+1));
	matrix_range<matrix<double>>Ayf2(*Ain,range(2*N,4*N),range(N+2+N+1,N+2+N+2));
	matrix_range<matrix<double>>by(*bin,  range(2*N,4*N),range(0      ,1));

	matrix_range<matrix_range<matrix<double>>>Axu1(Axu,  range(0,N),  range(0,N));
	matrix_range<matrix_range<matrix<double>>>Axu2(Axu,  range(N,2*N),range(0,N));
	matrix_range<matrix_range<matrix<double>>>Axf11(Axf1,range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>Axf12(Axf1,range(N,2*N),range(0,1));
	matrix_range<matrix_range<matrix<double>>>Axf21(Axf2,range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>Axf22(Axf2,range(N,2*N),range(0,1));
	matrix_range<matrix_range<matrix<double>>>bx1(bx,    range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>bx2(bx,    range(N,2*N),range(0,1));

	Axu1 = -Zu;     Axf11 = U2+U3+0.5*U4;   Axf21 = 0.5*U4;     bx1 =  prod(Zx,x0) - U0*(min(xf0,xs0)-0.05) - U1*(xf0-0.05) - (U2+U3)*(-0.05) - U4*(-0.05);
	Axu2 =  Zu;		Axf12 =-U2-U3-0.5*U4;	Axf22 =-0.5*U4;     bx2 = -prod(Zx,x0) + U0*(max(xf0,xs0)+0.05) + U1*(xf0+0.05) + (U2+U3)*(+0.05) + U4*(+0.05);

	matrix_range<matrix_range<matrix<double>>>Ayu1(Ayu,  range(0,N),  range(0,N));
	matrix_range<matrix_range<matrix<double>>>Ayu2(Ayu,  range(N,2*N),range(0,N));
	matrix_range<matrix_range<matrix<double>>>Ayf11(Ayf1,range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>Ayf12(Ayf1,range(N,2*N),range(0,1));
	matrix_range<matrix_range<matrix<double>>>Ayf21(Ayf2,range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>Ayf22(Ayf2,range(N,2*N),range(0,1));
	matrix_range<matrix_range<matrix<double>>>by1(by,    range(0,N),  range(0,1));
	matrix_range<matrix_range<matrix<double>>>by2(by,    range(N,2*N),range(0,1));

	Ayu1 = -Zu;     Ayf11 = U2+U3+0.5*U4;   Ayf21 = 0.5*U4;     by1 =  prod(Zx,y0) - U0*(min(yf0,ys0)-0.05) - U1*(yf0-0.05) - (U2+U3)*(-0.05) - U4*(-0.05);
	Ayu2 =  Zu;		Ayf12 =-U2-U3-0.5*U4;	Ayf22 =-0.5*U4;     by2 = -prod(Zx,y0) + U0*(max(yf0,ys0)+0.05) + U1*(yf0+0.05) + (U2+U3)*(+0.05) + U4*(+0.05);
    
    // =============step distance constraints=============

    matrix_range<matrix<double>>Astep(*Ain,range(4*N,4*N+8),range(0,2*(N+2)));
    matrix_range<matrix<double>>bstep(*bin,range(4*N,4*N+8),range(0,1));
    
    Astep(0,N+0) =  1.0;    Astep(0,N+1) =  0.0;    bstep(0,0) =  xf0+0.3;
    Astep(1,N+0) = -1.0;    Astep(1,N+1) =  0.0;    bstep(1,0) = -xf0+0.3;
    Astep(2,N+0) = -1.0;    Astep(2,N+1) =  1.0;    bstep(2,0) =     +0.3;
    Astep(3,N+0) =  1.0;    Astep(3,N+1) = -1.0;    bstep(3,0) =     +0.3; 
    
    Astep(4,N+2+N+0) =  1.0;    Astep(4,N+2+N+1) =  0.0;    bstep(4,0) =  yf0+0.3;
    Astep(5,N+2+N+0) = -1.0;    Astep(5,N+2+N+1) =  0.0;    bstep(5,0) = -yf0+0.3;
    Astep(6,N+2+N+0) = -1.0;    Astep(6,N+2+N+1) =  1.0;    bstep(6,0) =     +0.3;
    Astep(7,N+2+N+0) =  1.0;    Astep(7,N+2+N+1) = -1.0;    bstep(7,0) =     +0.3; 

	// =============foot placement constraints=============
    
 //   matrix_range<matrix<double>>Aside(*Ain,range(4*N+8,4*N+8+2),range(0,2*(N+2)));
 //   matrix_range<matrix<double>>bside(*bin,range(4*N+8,4*N+8+2),range(0,1));

	//if (ys0<yf0){
	//	Aside(0,N+2+N+0) =  1.0;    Aside(0,N+2+N+1) =  0.0;    bside(0,0) =  yf0-0.15;
	//	Aside(1,N+2+N+0) =  1.0;    Aside(1,N+2+N+1) = -1.0;    bside(1,0) =  -0.15;
	//}else{
	//	Aside(0,N+2+N+0) =  1.0;    Aside(0,N+2+N+1) =  0.0;    bside(0,0) = -yf0-0.15;
	//	Aside(1,N+2+N+0) = -1.0;    Aside(1,N+2+N+1) =  1.0;    bside(1,0) =  -0.15;
	//}

//     matrix_range<matrix<double>>AA(*Ain,range(0,4*N+8),range(0,2*N+4));
//     printf("\n");
//     for(i=0;i<4*N+8;i++){
//         for(j=0;j<2*N+4;j++)    printf("% f ",AA(i,j));
//         printf("\n");
//     }

}


void preview_simulate(double *x, double L, double T, int N, vector<double> u, double *traj){
	int i,j;
	matrix<double>A(3,3);
	matrix<double>B(3,1);
	matrix<double>C(1,3);
	matrix<double>P(3*N,3+N);
	matrix<double>Z(N,3+N);

	matrix<double>x0(3,1);
	matrix<double>y0(3,1);
	x0(0,0) = x[0];
	x0(1,0) = x[1];
	x0(2,0) = x[2];
	double xf0 = x[3];
	y0(0,0) = x[4];
	y0(1,0) = x[5];
	y0(2,0) = x[6];
	double yf0 = x[7];
	double xs0 = x[8];
	double ys0 = x[9];

	A(0,0) = 1.0; A(0,1) = T;   A(0,2) = T*T*0.5;
	A(1,0) = 0.0; A(1,1) = 1.0; A(1,2) = T;
	A(2,0) = 0.0; A(2,1) = 0.0; A(2,2) = 1.0;

	B(0,0) = T*T*T/6.0;
	B(1,0) = T*T*0.5;
	B(2,0) = T;

	C(0,0) = 1.0;	C(0,1) = 0.0;	C(0,2) = -L/9.81;

	P.clear();
	Z.clear();

	preview_recurse(&P,&Z,A,B,C,0,N);

	// position
	matrix_slice<matrix<double>> Ppx(P,slice(0,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Ppu(P,slice(0,3,N),slice(3,1,N));
	// velocity
	matrix_slice<matrix<double>> Pvx(P,slice(1,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Pvu(P,slice(1,3,N),slice(3,1,N));
	// acceleration
	matrix_slice<matrix<double>> Pax(P,slice(2,3,N),slice(0,1,3));
	matrix_slice<matrix<double>> Pau(P,slice(2,3,N),slice(3,1,N));
	// zmp
	matrix_slice<matrix<double>> Zx(Z,slice(0,1,N),slice(0,1,3));
	matrix_slice<matrix<double>> Zu(Z,slice(0,1,N),slice(3,1,N));

	matrix<double> px(N,1);
	matrix<double> vx(N,1);
	matrix<double> ax(N,1);
	matrix<double> py(N,1);
	matrix<double> vy(N,1);
	matrix<double> ay(N,1);
	matrix<double> ux(N,1);
	matrix<double> uy(N,1);

	for (i=0;i<N;i++){
		ux(i,0) = u(i);
		uy(i,0) = u(N+2+i);
	}

	px = prod(Ppx,x0) + prod(Ppu,ux);
	py = prod(Ppx,y0) + prod(Ppu,uy);
	vx = prod(Pvx,x0) + prod(Pvu,ux);
	vy = prod(Pvx,y0) + prod(Pvu,uy);
	ax = prod(Pax,x0) + prod(Pau,ux);
	ay = prod(Pax,y0) + prod(Pau,uy);

	for(i=0;i<N;i++){
		traj[6*i+0] = px(i,0);
		traj[6*i+1] = py(i,0);
		traj[6*i+2] = vx(i,0);
		traj[6*i+3] = vy(i,0);
		traj[6*i+4] = ax(i,0);
		traj[6*i+5] = ay(i,0);
	}
}


double preview2_control(double *x, double L, double T, int N, double Tstep, double Tdsp, double *step, double *traj){
	// x = [x,dx,ddx,xf0,y,dy,ddy,yf0,xs0,ys0]
	int i,j;
	double fval;
	int dir=1;		
    int M = 0;

	double xf0 = x[3];
	double yf0 = x[7];
	double xs0 = x[8];
	double ys0 = x[9];

	// =============No equality constraints=============
	matrix<double> CE(2*(N+2),0);
	vector<double>ce0(0);
	
	CE.clear();
	ce0.clear();

	// =============Inequality constraints=============    
    
	matrix<double> A( 4*N+8,2*(N+2));
	vector<double> b( 4*N+8);
	matrix<double> b2(4*N+8,1);
// 	matrix<double> A( 0,2*(N+2));
// 	vector<double> b( 0);
// 	matrix<double> b2(0,1); 

	// =============Objective function=============
	matrix<double>Q(2*(N+2),2*(N+2));
	matrix<double>p(2*(N+2),1);
	vector<double>f(2*(N+2));

	preview_init(&Q, &p, &A, &b2, x, L, T, N, Tstep, Tdsp);

	for(i=0;i<N+2;i++){
		f(i) = p(i,0);
		f(N+2+i) = p(N+2+i,0);
	}

	for(i=0;i<b2.size1();i++)	b(i) = b2(i,0);

	// =============Solve=============
	vector<double>u(2*(N+2));
//     matrix<double>Q2(2*(N+1),2*(N+1));    
//     Q2 = 0.5*(trans(Q)+Q);
    
	fval = solve_quadprog(Q, f,  CE, ce0,  -trans(A), b, u);

	//std::cout << u << std::endl;

	step[0] = u(N);
	step[1] = u(N+2+N);
	step[2] = u(N+1);
	step[3] = u(N+2+N+1);    

	printf("fval = % 5f step = %f %f\n",fval,step[0],step[1]);


	//**********************

	if (traj!=NULL)
	preview_simulate(x, L, T, N, u, traj);

	return fval;
}


#endif
