#include "preview0.h"
#include "preview0-force.h"
#include "preview2-time.h"
#include "preview2-time-walk.h"
#include "preview2-force.h"


#define _WIN32_WINNT 0x0500
#pragma comment(lib,"winmm.lib")

#include <iostream>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <math.h>

#include "preview_common.h"

// the shared memory
static preview_t pDataArray[MAX_THREADS];
static preview_t *pData;

// events for synchronization
static HANDLE hInputReadyEvent[MAX_THREADS];
static HANDLE hOutputReadyEvent[MAX_THREADS];
static HANDLE hWaitingEvent[MAX_THREADS];
static HANDLE hInitializedEvent[MAX_THREADS];

// threads to handle processing
static HANDLE hThread[MAX_THREADS];
static DWORD lpThreadID[MAX_THREADS];

static double server_start_time = 0.0;

static double get_time()
{
  return (double) timeGetTime()/1000.0;
}

#define SERVER_TIME	(get_time()-server_start_time)

#define PREVIEW_SERVER_DEBUG	0

static double newest_time = -1;

//#define USE_MULTIPLE_PREVIEW2

#ifdef USE_MULTIPLE_PREVIEW2
#define N_STEP_TIMES 5
double step_times_set[N_STEP_TIMES] = {STEP_TIME-0.1,  STEP_TIME, STEP_TIME+0.1};


void Preview2Thread(void *pThis){
	preview_data_t *share = (preview_data_t*)pThis;
	int id = share->id;
	preview_data_t local;
	int i,j;
	Preview2 preview2;

	//WaitForSingleObject(share->hMutex,INFINITE);
		for(i=0;i<10;i++)	x0[i] = share->x0[i];
		step_time = share->step_time;
		t = share->t;

		cost = preview2.run(x0, 1.0, t, STEP_T, STEP_N, step_time, STEP_DSP_T, step_des,(double*)traj, ttraj);

		for(i=0;i<100;i++){
			for(j=0;j<6;j++){
				share->traj[i][j] = traj[i][j];
			}
			share->ttraj[i] = ttraj[i];
		}
		for(i=0;i<2;i++)	share->step_des[i] = step_des[i];
		share->cost = cost;
	//ReleaseMutex(share->hMutex);
}

double preview2_best(double *x0, double t, double *step_des, double *step_time, double *traj, double *ttraj){
	double best_cost=1.0e20;
	int best_idx=-1;
	int i,j;
	Preview2Data data[N_STEP_TIMES];
	HANDLE ghThread[N_STEP_TIMES];
	DWORD lpThreadID[N_STEP_TIMES];

	// start the threads
	for(i=0;i<N_STEP_TIMES;i++){
		for(j=0;j<10;j++)	data[i].x0[j] = x0[j];
		data[i].t = t;
		data[i].step_time = step_times_set[i];

		// create the Mutexes
		//ghMutex[i] = CreateMutex(NULL,0,NULL);
		//data[i].hMutex = ghMutex[i];

		// start the thread
		ghThread[i] = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)Preview2Thread,&data[i],0,&lpThreadID[i]);
	}

	// wait for them all to finish
	WaitForMultipleObjects(N_STEP_TIMES,ghThread,TRUE,INFINITE);

	// find the result with lowest cost
	for(i=0;i<N_STEP_TIMES;i++){
		if (data[i].cost<best_cost){
			best_idx = i;
			best_cost = data[i].cost;
		}
	}

	if (best_idx==-1){
		printf("ERROR\n");
	}else{
		printf("best_idx = %i | cost = %f %f %f\n",best_idx,data[0].cost,data[1].cost,data[2].cost);
	}

	// copy the data from the best cost trajectory
	for(i=0;i<100;i++){
		for(j=0;j<6;j++){
			traj[6*i+j] = data[best_idx].traj[i][j];
		}
		ttraj[i] = data[best_idx].ttraj[i];
	}
	for(i=0;i<4;i++)	step_des[i] = data[best_idx].step_des[i];
	step_time[0] = data[best_idx].step_time;
	return data[best_idx].cost;

}

#endif


static void previewThread( void *pThis ){
	preview_t *share = (preview_t*)pThis;
	int id = share->id;
	Preview2 preview2(STEP_N);
	Preview0 preview0(STEP_N);
	Preview0force preview0force(STEP_N);
	Preview2walk preview2walk(STEP_N);
	Preview2force preview2force(STEP_N);
	preview_data_t data;
	double thread_start_time;

	//for(id=0;id<MAX_THREADS;id++){
	//	if (share->lpThreadID==GetCurrentThreadId())	break;
	//}
	if (id>0)	WaitForMultipleObjects(id,hInitializedEvent,TRUE,INFINITE);	// wait to initialize one at a time

	preview0force.run(NULL, 1.0, STEP_T, STEP_N, NULL, NULL);	// initializes the function	
	preview2force.run(NULL, 1.0, 0, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, NULL,NULL,NULL);	
	//preview2walk.run(NULL, 1.0, 0, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, NULL, NULL,NULL,NULL);
	//preview2walk.run(NULL, 1.0, 0, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, NULL, NULL,NULL,NULL);

	SetEvent(hInitializedEvent[id]);

	if(PREVIEW_SERVER_DEBUG) printf("Thread %i spawned with id = %d\n",id,GetCurrentThreadId());

	while(1){
		
		// wait for the event to read the input
		WaitForSingleObject(hInputReadyEvent[id],INFINITE);

		ResetEvent(hInputReadyEvent[id]);

		memcpy(&data,&share->data,sizeof(preview_data_t));

		if(PREVIEW_SERVER_DEBUG) printf("%7.3f: Server: Thread %i running (type=%i), time=% 5.3f\n",SERVER_TIME,id,data.type,data.start_time);

		thread_start_time = get_time();

		switch(data.type){
			case PREVIEW0_RHC:
				data.cost = preview0force.run(data.x0, 1.0, STEP_T, STEP_N, (double*)data.traj, data.ttraj);
				data.step_time = 0;
				data.step_des[0] = data.x0[8];
				data.step_des[1] = data.x0[9];
				break;

//#ifdef USE_MULTIPLE_PREVIEW2
//			case PREVIEW2_RHC:
//				cost = preview2_best(x0,t,step_des,&step_time,(double*)traj,ttraj);
//				break;
//#else
			case PREVIEW2_RHC:				
				//data.cost = preview2.run(data.x0, 1.0, data.t, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, data.step_des,(double*)data.traj, data.ttraj);
				data.cost = preview2force.run(data.x0, 1.0, data.t, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, data.step_des,(double*)data.traj, data.ttraj);
				data.step_time = STEP_TIME;
				break;
//#endif 

			//case PREVIEW2_WALK_RHC:
			//	data.cost = preview2walk.run(data.x0, 1.0, data.t, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, data.vdes, data.step_des,(double*)data.traj, data.ttraj);
			//	data.step_time = STEP_TIME;
			//	break;

			//case PREVIEW3_RHC:
			//	cost = preview3[id].run(x0, 1.0, t, STEP_T, STEP_N, STEP_TIME, STEP_DSP_T, step_des,(double*)traj);
			//	break;

			default:
				printf("########UNDEFINED PREVIEW TYPE!############\n");
				break;

		}
		
		if(PREVIEW_SERVER_DEBUG) printf("%7.3f: Server: Thread %i Finished Cost = %f Elapsed = %5.4f\n",SERVER_TIME,id,data.cost,get_time()-thread_start_time);

		memcpy(&share->data,&data,sizeof(preview_data_t));			

		SetEvent(hOutputReadyEvent[id]);
		//SetEvent(hWaitingEvent[id]);


		// continue the while loop and wait for next event pulse
	}
}

static int preview_reinit_cpp(){
	int i;
	for(i=0;i<MAX_THREADS;i++){
		SetEvent(hWaitingEvent[i]);
		ResetEvent(hInputReadyEvent[i]);
		ResetEvent(hOutputReadyEvent[i]);
	}
	newest_time = -1;
	return 1;
}	

static void preview_input(void *pThis){
	int cur_thread;
	preview_data_t *data = (preview_data_t*)pThis;

	// look for a waiting thread
	for(cur_thread=0;cur_thread<MAX_THREADS;cur_thread++){
		if ((WaitForSingleObject(hWaitingEvent[cur_thread],0) == WAIT_OBJECT_0) &&
			(WaitForSingleObject(hInputReadyEvent[cur_thread],0) != WAIT_OBJECT_0) && 
			(WaitForSingleObject(hOutputReadyEvent[cur_thread],0) != WAIT_OBJECT_0))	break;	
	}

	if (cur_thread>=MAX_THREADS){	
		if(PREVIEW_SERVER_DEBUG) printf("Server: No available threads!\n");
	}else{
		pData = &pDataArray[cur_thread];

		if(PREVIEW_SERVER_DEBUG) printf("%7.3f: Server: Thread %i assigned to data\n",SERVER_TIME,cur_thread);

		memcpy(&pData->data,data,sizeof(preview_data_t));
		
		SetEvent(hInputReadyEvent[cur_thread]);
		ResetEvent(hWaitingEvent[cur_thread]);
	}
}

int preview_recv_cpp(preview_data_t *data){
	static preview_data_t temp;

	memcpy(&temp,data,sizeof(preview_data_t));

	if(1){
		preview_input((void*)&temp);
	}else{
		CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)preview_input,&temp,0,NULL);
	}
	
	//if(PREVIEW_SERVER_DEBUG) printf("% 7.3f: Server: Recv\n",SERVER_TIME);	

	return 1;
}


int preview_send_cpp(preview_data_t *data){
	// check if any threads are done
	int newest_thread = MAX_THREADS;
	int cur_thread;

	for(cur_thread=0;cur_thread<MAX_THREADS;cur_thread++){
		if (WaitForSingleObject(hOutputReadyEvent[cur_thread],0) == WAIT_OBJECT_0){
			pData = &pDataArray[cur_thread];
			if (pData->data.start_time>newest_time){
				newest_thread = cur_thread;
				newest_time = pData->data.start_time;
				//break;
			}
			if(PREVIEW_SERVER_DEBUG) printf("%7.3f: Server: Thread %i Processed output, time=% 5.3f\n",SERVER_TIME,cur_thread,data->start_time);
			SetEvent(hWaitingEvent[cur_thread]);	
			ResetEvent(hOutputReadyEvent[cur_thread]);
		}
	}

	if (newest_thread<MAX_THREADS){
		pData = &pDataArray[newest_thread];
		memcpy(data,&pData->data,sizeof(preview_data_t));
		if(PREVIEW_SERVER_DEBUG) printf("%7.3f: Server: Thread %i Writing output data, time=% 5.3f\n",SERVER_TIME,newest_thread,data->start_time);
	}else	return 0;

	return 1;
}

static void fake_client(void *pThis){
	preview_data_t fake;
	for(int i=0;i<10;i++)	fake.x0[i] = 0.0;
	fake.t = 0.0;
	fake.type = PREVIEW0_RHC;
	while(1){
		int timestep = ceil((SERVER_TIME)/0.0025);
		int freq = ceil(1.0/0.0025/RHC_FREQ);
		if (timestep%freq==0){
			
			fake.start_time = SERVER_TIME;

			preview_recv_cpp(&fake);

			Sleep(3);
		}
	}
}

static int preview_init_cpp(){
	int i;

	printf("Creating Preview Worker Threads...");

	for(i=0;i<MAX_THREADS;i++){
		// initialize the shared memory variables
		pDataArray[i].id = i;

		// create mutex and events
		hInputReadyEvent[i] = CreateEvent(NULL, TRUE, FALSE, NULL);
		hOutputReadyEvent[i] = CreateEvent(NULL, TRUE, FALSE, NULL);
		hWaitingEvent[i] = CreateEvent(NULL, TRUE, TRUE, NULL);
		hInitializedEvent[i] = CreateEvent(NULL, TRUE, FALSE, NULL);

		// start the thread
		hThread[i] = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)previewThread,&pDataArray[i],0,&lpThreadID[i]);
		SetThreadPriority(hThread[i],THREAD_PRIORITY_ABOVE_NORMAL);
		SetPriorityClass(hThread[i], HIGH_PRIORITY_CLASS);
		SetThreadIdealProcessor(hThread[i],2);
	}

	//Sleep(1000);
	//CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)fake_client,NULL,0,NULL);

	printf("done\n");

	server_start_time = get_time();

	return 1;
}



extern "C"{
	int preview_init(void){return preview_init_cpp();}
	int preview_reinit(void){return preview_reinit_cpp();}
	int preview_send(preview_data_t *data){return preview_recv_cpp(data);}
	int preview_recv(preview_data_t *data){return preview_send_cpp(data);}
}