#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include "gsl_util.h"

void gsl_matrix_vertcat(gsl_matrix *A, gsl_matrix *B, gsl_matrix *C){
	gsl_matrix_view a = gsl_matrix_submatrix(C,0,0,A->size1,A->size2);
	gsl_matrix_view b = gsl_matrix_submatrix(C,A->size1,0,B->size1,B->size2);
	gsl_matrix_memcpy(&a.matrix,A);
	gsl_matrix_memcpy(&b.matrix,B);
}

void gsl_matrix_horzcat(gsl_matrix *A, gsl_matrix *B, gsl_matrix *C){
	gsl_matrix_view a = gsl_matrix_submatrix(C,0,0,A->size1,A->size2);
	gsl_matrix_view b = gsl_matrix_submatrix(C,0,A->size2,B->size1,B->size2);
	gsl_matrix_memcpy(&a.matrix,A);
	gsl_matrix_memcpy(&b.matrix,B);
}

//void gsl_pseudosolve_vector(double *A, double *b, int trans, int m, int n, double *x){
//	gsl_matrix *U = gsl_matrix_alloc(m,n);
//	gsl_vector *S = gsl_vector_alloc(n);
//	gsl_matrix *V = gsl_matrix_alloc(n,n);
//	gsl_vector *work = gsl_vector_alloc(n);
//	gsl_vector *bb = gsl_vector_alloc(m);
//	
//	gsl_matrix_view Av = gsl_matrix_view_array(A,m,n);
//	gsl_matrix_memcpy(U,&Av.matrix);
//
//    gsl_linalg_SV_decomp(U,V,S,work);
//
//	gsl_vector_view bv = gsl_vector_view_array(b,m);
//	gsl_vector_view xv = gsl_vector_view_array(x,n);
//
//	gsl_vector_memcpy(bb,&bv.vector);
//
//	gsl_linalg_SV_solve(U,V,S,bb,&xv.vector);
//
//	gsl_matrix_free(U);
//	gsl_vector_free(S);
//	gsl_matrix_free(V);
//	gsl_vector_free(work);
//	gsl_vector_free(bb);
//}

void gsl_pseudosolve_vector(double *A, double *b, int trans, int m, int n, double *x){
 	gsl_matrix *AA = gsl_matrix_calloc(n,n);
 	gsl_vector *bb = gsl_vector_calloc(n);
	gsl_permutation *p = gsl_permutation_alloc(n);
     
	if (trans){
		gsl_matrix_view Av = gsl_matrix_view_array(A,n,m);
		gsl_vector_view bv = gsl_vector_view_array(b,m);      
		gsl_blas_dgemm(CblasNoTrans,CblasTrans,1.0,&Av.matrix,&Av.matrix,0.0,AA);
		gsl_blas_dgemv(CblasNoTrans,1.0,&Av.matrix,&bv.vector,0.0,bb);
	}else{
		gsl_matrix_view Av = gsl_matrix_view_array(A,m,n);
		gsl_vector_view bv = gsl_vector_view_array(b,m);      
		gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,&Av.matrix,&Av.matrix,0.0,AA);
		gsl_blas_dgemv(CblasTrans,1.0,&Av.matrix,&bv.vector,0.0,bb);
	}

	int signum;
	gsl_vector_view xv = gsl_vector_view_array(x,n);
	gsl_linalg_LU_decomp(AA,p,&signum);
	gsl_linalg_LU_solve(AA,p,bb,&xv.vector);

	gsl_matrix_free(AA);
	gsl_vector_free(bb);
	gsl_permutation_free(p);
}

void gsl_pseudosolve_matrix(double *A, double *B, int trans, int m, int n, int r, double *X){
 	gsl_matrix *AA = gsl_matrix_calloc(n,n);
 	gsl_matrix *BB = gsl_matrix_calloc(n,r);
     
     if (trans){
         gsl_matrix_view Av = gsl_matrix_view_array(A,n,m);
         gsl_matrix_view Bv = gsl_matrix_view_array(B,r,m);      
         gsl_blas_dgemm(CblasNoTrans,CblasTrans,1.0,&Av.matrix,&Av.matrix,0.0,AA);
         gsl_blas_dgemm(CblasNoTrans,CblasTrans,1.0,&Av.matrix,&Bv.matrix,0.0,BB);
     }else{
         gsl_matrix_view Av = gsl_matrix_view_array(A,m,n);
         gsl_matrix_view Bv = gsl_matrix_view_array(B,m,r);      
         gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,&Av.matrix,&Av.matrix,0.0,AA);
         gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,&Av.matrix,&Bv.matrix,0.0,BB);
     }
}