#ifndef PRIMUS_COMMON
#define PRIMUS_COMMON

#include "SDModel.h"
#include <gsl/gsl_linalg.h>

class Primus{
public:
	Primus(SDModel*model);
	~Primus();
	SDModel *m;
	double *JL,*JR;
	double PL[6],PR[6];
	double *state,*des_state;
	gsl_matrix_view gsl_JL,gsl_JR;
	gsl_matrix *gsl_J;
};

//-------- requires call to set_state prior to exec --------//

// returns centers of feet and orientation (and jacobian)
void primus_feet(SDModel *m, double *fl, double *Jl, double *fr, double *Jr);

// returns vector of 3D poses of feet corners (and jacobian)
void primus_feetcorners(SDModel *m, double *fl, double *Jl, double *fr, double *Jr);

// COM and velocity w.r.t. each foot
void primus_com_wrt_feet(SDModel *m, double *cl, double *Jcl, double *cr, double *Jcr);

// position of point where hoses contact the robot
void primus_hose_contact(SDModel *m, double *p, double *J);

// gravity compensation inverse dynamics
void primus_gravcomp(SDModel *m, double *X, double *U);

void primus_forward_kinematics(SDModel *m,
							   double *state,
							   double *com,
							   double *pL,
							   double *pR,
							   double *T);


//-------- performs multiple calls to set_state ------------//

void primus_com_ik(SDModel *m, double *q, double *pLd_des, double *pRd_des, double *dpdes, double *dTdes, double *dq);

//-------- requires call to sdderiv prior to exec ----------//




#endif