/*****************************************************************************/
#define MAIN


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef WIN32
#include <windows.h>
#else
#include <time.h>
#endif

#include "main.h"
#include "main2.h"

//#define RUN_MULTIPLE_PUSH_EXPERIMENT


/*****************************************************************************/
/*****************************************************************************/

extern SIM sim;

double last_time = 0;

/*****************************************************************************/
/*****************************************************************************/
/************************************************************************/

#ifdef WIN32
double get_time()
{
  return (double) timeGetTime()/1000.0;
}
#else
double get_time()
{
  struct timespec the_tp;

  clock_gettime( CLOCK_MONOTONIC, &the_tp );
  return ((double) (the_tp.tv_sec)) + 1.0e-9*the_tp.tv_nsec;
}
#endif

/*****************************************************************************/

main( int argc, char **argv )
{
  PARAMETER *params;
  int n_parameters;
  int i;
  double the_time;
  int success=1;
  FILE *state_file;
#ifdef RUN_MULTIPLE_PUSH_EXPERIMENT
  int n1,n2,n3;
  FILE *push_file = fopen("push_file","w");
#endif
	double push_x, push_y, push_t;
	double push_mag, push_ang;

  

#ifdef WIN32
  last_time = get_time();
#else
  last_time = get_time();
#endif


  init_dynamics_parameters( &sim );
  init_controller_parameters( &sim );
  sim.rand_scale = 0;
  sim.controller_print = 1;

  /* Parameter file argument? */
  if ( argc > 1 )
    {
      params = read_parameter_file( argv[1] );
      n_parameters = process_parameters( params, &sim, 1 );
      if ( n_parameters > MAX_N_PARAMETERS )
	{
	  fprintf( stderr, "Too many parameters %d > %d\n",
		   n_parameters, MAX_N_PARAMETERS );
	  exit( -1 );
	}
    }

  init_controller( &sim );
  init_data( &sim );

#ifdef RUN_MULTIPLE_PUSH_EXPERIMENT
  for(n2=0;n2<15;n2++){
  for(n3=0;n3<5;n3++){
  for(n1=0;n1<8;n1++){

		push_x = (100.0 + 300.0*(n1/7.0))*cos(2*3.14159*n2/15.0);
		push_y = (100.0 + 300.0*(n1/7.0))*sin(2*3.14159*n2/15.0);
		push_t = 2.0 + 0.2*(n3/5.0);
		printf("\n\n\n#### %f %f %f ####\n\n\n",push_x,push_y,push_t);
		success=1;
#else
	push_mag = 50;
	push_ang = 0;
	push_t = 2.0;
	push_x = push_mag*cos(push_ang);
	push_y = push_mag*sin(push_ang);
#endif

	  state_file = fopen("state_start","rb");
		fread(&sim,sizeof(SIM),1,state_file);
	  fclose(state_file);

	  sim.time=0.0;
	  sim.cstate_start_time=0.0;
	  sim.duration = 7.0;
	  sim.controller_state = 1;

	  for( i = 0; sim.time < sim.duration; i++ )
		{
		  if ( (i % 1000) == 0 )
			{
		  
			  printf( "%g: %g %g %g\n", sim.time,
			  sim.com[XX], sim.com[YY], sim.com[ZZ] );
		  
			}

//#ifdef RUN_MULTIPLE_PUSH_EXPERIMENT
		  /* Perturbation */
		  if ( push_t <= sim.time && sim.time < push_t+2.0 ){
			sim.torso_perturbation_force[XX] = push_x;
			sim.torso_perturbation_force[YY] = push_y;
		  } else{
			sim.torso_perturbation_force[XX] = 0.0;
			sim.torso_perturbation_force[YY] = 0.0;
		  }
//#endif

		  controller( &sim );
#ifndef RUN_MULTIPLE_PUSH_EXPERIMENT
		  sim.cstate = sim.controller_state;

		  sim.realtime = get_time() - last_time;

		  save_data( &sim );
#endif
		  if ( sim.status == CRASHED ){
			  success = 0;
			break;
		  }
		  integrate_one_time_step( &sim );
	  }

#ifdef RUN_MULTIPLE_PUSH_EXPERIMENT
		fprintf(push_file,"%f %f %f %i\n",push_x,push_y,push_t,success); 
  }
  }
  }
#endif

#ifdef RUN_MULTIPLE_PUSH_EXPERIMENT
  fclose(push_file);
#endif

#ifndef RUN_MULTIPLE_PUSH_EXPERIMENT
    write_the_mrdplot_file( &sim );
  state_file = fopen("sim_state","wb");
	fwrite(&sim,sizeof(SIM),1,state_file);
  fclose(state_file);
#endif
  
#ifdef WIN32
  the_time = get_time();
  printf( "time: %g\n", the_time - last_time );
#else
  the_time = get_time();
  printf( "time: %g\n", the_time - last_time );
#endif
}

/*****************************************************************************/
