#include "ben_common.h"

extern "C"{


#include <stdio.h>
#include <math.h>
#include <stdlib.h>
//#include "main.h"
#include "state_machine.h"
#include "sdlib.h"
#include "trajectory.h"

#define ZZ 2
#define LEFT	0
#define RIGHT	1

// NOTE: Multithreading only works in RHC mode currently
#define USE_MULTITHREADING
#define USE_PREVIEW_SOCKETS

#define STOP_AFTER_RECOVERED

//#define USE_MULTITHREADING_ID

#ifdef USE_MULTITHREADING
	#define _WIN32_WINNT 0x0500
	#include <windows.h>
	#include <process.h>
#endif
	
#define USE_RECEDING_HORIZON

	//static double traj[100][6]={0.0};
DOFS com_traj[2];
DOFS comdd_traj[2];
DOFS foot_traj[3];

#include "preview_common.h"

static double get_time()
{
  return (double) timeGetTime()/1000.0;
}

static double server_start_time;

#define SERVER_TIME	(get_time()-server_start_time)

static void process_preview(PRIMUS_t *s){
	int i,j;
	static preview_data_t pData;
	static double newest_time = -1;
	int spline_type = CUBIC_SPLINE;

	if (s->time==0.0){
		newest_time = -1;
		return; // can't possibly have an answer
	}

	if (!preview_recv(&pData))	return;

	if (pData.start_time>newest_time && pData.cost<100){
		newest_time = pData.start_time;
	}else return;

	printf("% 6.3f: State Machine: Received start_time = %6.3f Cost=%5.4f Type = %i\n",s->time,pData.start_time,pData.cost,pData.type);

	clear_all_knots(com_traj,2);
	clear_all_knots(comdd_traj,2);

	switch(pData.type){

		case PREVIEW0_RHC:

			add_knot_point( com_traj, 0, spline_type, pData.start_time + PREVIEW0_RHC_DELAY, 
				pData.x0[0], pData.x0[1], pData.x0[2] );
			add_knot_point( com_traj, 1, spline_type, pData.start_time + PREVIEW0_RHC_DELAY, 
				pData.x0[4], pData.x0[5], pData.x0[6] );

			add_knot_point( comdd_traj, 0, LINEAR_SPLINE, pData.start_time + PREVIEW0_RHC_DELAY, 
				pData.x0[2], pData.traj[0][6], 0.0 );
			add_knot_point( comdd_traj, 1, LINEAR_SPLINE, pData.start_time + PREVIEW0_RHC_DELAY, 
				pData.x0[6], pData.traj[0][7], 0.0 );

			for(i=0;i<STEP_N;i++){
				add_knot_point( com_traj, 0, spline_type, pData.start_time + PREVIEW0_RHC_DELAY+pData.ttraj[i], 
					pData.traj[i][0], pData.traj[i][2], pData.traj[i][4] );
				add_knot_point( com_traj, 1, spline_type, pData.start_time + PREVIEW0_RHC_DELAY+pData.ttraj[i], 
					pData.traj[i][1], pData.traj[i][3], pData.traj[i][5] );

				if (i<STEP_N-1){
					add_knot_point( comdd_traj, 0, LINEAR_SPLINE, pData.start_time + PREVIEW0_RHC_DELAY+pData.ttraj[i], 
						pData.traj[i][4], pData.traj[i+1][6], 0.0 );
					add_knot_point( comdd_traj, 1, LINEAR_SPLINE, pData.start_time + PREVIEW0_RHC_DELAY+pData.ttraj[i], 
						pData.traj[i][5], pData.traj[i+1][7], 0.0 );
				}
			}


			break;	// case PREVIEW0_RHC


		case PREVIEW2_RHC:
			
			add_knot_point( com_traj, 0, spline_type, pData.start_time + PREVIEW2_RHC_DELAY, 
				pData.x0[0], pData.x0[1], pData.x0[2] );
			add_knot_point( com_traj, 1, spline_type, pData.start_time + PREVIEW2_RHC_DELAY, 
				pData.x0[4], pData.x0[5], pData.x0[6] );

			add_knot_point( comdd_traj, 0, LINEAR_SPLINE, pData.start_time + PREVIEW2_RHC_DELAY, 
				pData.x0[2], pData.traj[0][6], 0.0 );
			add_knot_point( comdd_traj, 1, LINEAR_SPLINE, pData.start_time + PREVIEW2_RHC_DELAY, 
				pData.x0[6], pData.traj[0][7], 0.0 );

			for(i=0;i<STEP_N;i++){
				add_knot_point( com_traj, 0, spline_type, pData.start_time + PREVIEW2_RHC_DELAY+pData.ttraj[i], 
					pData.traj[i][0], pData.traj[i][2], pData.traj[i][4] );
				add_knot_point( com_traj, 1, spline_type, pData.start_time + PREVIEW2_RHC_DELAY+pData.ttraj[i], 
					pData.traj[i][1], pData.traj[i][3], pData.traj[i][5] );

				if (i<STEP_N-1){
					add_knot_point( comdd_traj, 0, LINEAR_SPLINE, pData.start_time + PREVIEW2_RHC_DELAY+pData.ttraj[i], 
						pData.traj[i][4], pData.traj[i+1][6], 0.0 );
					add_knot_point( comdd_traj, 1, LINEAR_SPLINE, pData.start_time + PREVIEW2_RHC_DELAY+pData.ttraj[i], 
						pData.traj[i][5], pData.traj[i+1][7], 0.0 );
				}
			}
			
			for(i=0;i<2;i++)	s->step_des[i] = pData.step_des[i];

			
			switch (s->controller_state){

				case STATE_STEPLEFT:
					clear_all_knots(foot_traj,3);

					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[RIGHT][0], s->footd_d[RIGHT][0], s->footdd_d[RIGHT][0] );
					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[0], 0.0, 0.0 );

					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[RIGHT][1], s->footd_d[RIGHT][1], s->footdd_d[RIGHT][1]);
					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[1], 0.0, 0.0 );

					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[RIGHT][2], s->footd_d[RIGHT][2], s->footdd_d[RIGHT][2] );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.5*pData.step_time, 
					  0+STEP_HEIGHT, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, pData.step_time+TOUCH_DELAY, 
					  0, 0.0, 0.0 );

					break;

				case STATE_STEPRIGHT:
					clear_all_knots(foot_traj,3);

					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[LEFT][0], s->footd_d[LEFT][0], s->footdd_d[LEFT][0] );
					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[0], 0.0, 0.0 );

					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[LEFT][1], s->footd_d[LEFT][1], s->footdd_d[LEFT][1]);
					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[1], 0.0, 0.0 );

					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, s->cstate_elapsed_time, 
					  s->foot_d[LEFT][2], s->footd_d[LEFT][2], s->footdd_d[LEFT][2] );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.5*pData.step_time, 
					  0+STEP_HEIGHT, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, pData.step_time+TOUCH_DELAY, 
					  0, 0.0, 0.0 );

					break;

				case STATE_LIFTLEFT:
				case STATE_SHIFTLEFT:
					clear_all_knots(foot_traj,3);

					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, 0.0, 
					  s->foot[RIGHT][0], 0.0, 0.0 );
					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[0], 0.0, 0.0 );

					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, 0.0, 
					  s->foot[RIGHT][1], 0.0, 0.0);
					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[1], 0.0, 0.0 );

					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.0, 
					  0.0, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.5*pData.step_time, 
					  0+STEP_HEIGHT, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, pData.step_time+TOUCH_DELAY, 
					  0, 0.0, 0.0 );

					break;

				case STATE_LIFTRIGHT:
				case STATE_SHIFTRIGHT:
					clear_all_knots(foot_traj,3);

					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, 0.0, 
					  s->foot[LEFT][0], 0.0, 0.0 );
					add_knot_point( foot_traj, 0, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[0], 0.0, 0.0 );

					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, 0.0, 
					  s->foot[LEFT][1], 0.0, 0.0);
					add_knot_point( foot_traj, 1, QUINTIC_SPLINE, pData.step_time, 
					  s->step_des[1], 0.0, 0.0 );

					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.0, 
					  0.0, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, 0.5*pData.step_time, 
					  0+STEP_HEIGHT, 0.0, 0.0 );
					add_knot_point( foot_traj, 2, QUINTIC_SPLINE, pData.step_time+TOUCH_DELAY, 
					  0, 0.0, 0.0 );

					break;
			}
			

			break;	// case PREVIEW2_RHC
	}

}


static void run_preview(PRIMUS_t *s, int TYPE){
	int i;
	static preview_data_t pData;
	double temp;

	process_preview(s);

	if (fmod(ceil((s->cstate_elapsed_time)/0.0025),ceil(1.0/0.0025/RHC_FREQ))==0.0){

#ifdef USE_STATE_ESTIMATOR
		pData.x0[0] = s->com_est[0];
		pData.x0[1] = s->comd_est[0];
		pData.x0[2] = s->comdd_d[0];
		pData.x0[4] = s->com_est[1];
		pData.x0[5] = s->comd_est[1];
		pData.x0[6] = s->comdd_d[1];
#else

		pData.x0[0] = s->com[0];
		pData.x0[1] = s->comd[0];
		pData.x0[2] = s->comdd_d[0];
		pData.x0[4] = s->com[1];
		pData.x0[5] = s->comd[1];
		pData.x0[6] = s->comdd_d[1];

		pData.x0[10] = 0*s->com_force_est[0]/s->total_mass;
		pData.x0[11] = 0*s->com_force_est[1]/s->total_mass;

		// use the feedforward comdd_d from the future trajectory if there is one
		// also offset by the current difference between the ff command and actual command
		switch(TYPE){

			case PREVIEW0_RHC:

				pData.x0[0] += PREVIEW0_RHC_DELAY*pData.x0[1] + 0.5*pData.x0[2]*PREVIEW0_RHC_DELAY*PREVIEW0_RHC_DELAY;
				pData.x0[1] += PREVIEW0_RHC_DELAY*pData.x0[2];
				pData.x0[4] += PREVIEW0_RHC_DELAY*pData.x0[5] + 0.5*pData.x0[6]*PREVIEW0_RHC_DELAY*PREVIEW0_RHC_DELAY;
				pData.x0[5] += PREVIEW0_RHC_DELAY*pData.x0[6];

				if(lookup_spline3_alt( comdd_traj, 0, s->time + PREVIEW0_RHC_DELAY, NULL,&pData.x0[2], &temp, &temp ))
					//pData.x0[2] += s->comdd_d[0] - s->comdd_ff[0];
				if(lookup_spline3_alt( comdd_traj, 1, s->time + PREVIEW0_RHC_DELAY, NULL,&pData.x0[6], &temp, &temp ))

				//if(lookup_spline3_alt( com_traj, 0, s->time + PREVIEW0_RHC_DELAY, NULL,&temp, &temp, &pData.x0[2] ))
				//	pData.x0[2] += s->comdd_d[0] - s->comdd_ff[0];
				//if(lookup_spline3_alt( com_traj, 1, s->time + PREVIEW0_RHC_DELAY, NULL,&temp, &temp, &pData.x0[6] ))
				//	pData.x0[6] += s->comdd_d[1] - s->comdd_ff[1];

				break;

			case PREVIEW2_RHC:

				pData.x0[0] += PREVIEW2_RHC_DELAY*pData.x0[1] + 0.5*pData.x0[2]*PREVIEW2_RHC_DELAY*PREVIEW2_RHC_DELAY;
				pData.x0[1] += PREVIEW2_RHC_DELAY*pData.x0[2];
				pData.x0[4] += PREVIEW2_RHC_DELAY*pData.x0[5] + 0.5*pData.x0[6]*PREVIEW2_RHC_DELAY*PREVIEW2_RHC_DELAY;
				pData.x0[5] += PREVIEW2_RHC_DELAY*pData.x0[6];

				if(lookup_spline3_alt( comdd_traj, 0, s->time + PREVIEW2_RHC_DELAY, NULL,&pData.x0[2], &temp, &temp ))
					//pData.x0[2] += s->comdd_d[0] - s->comdd_ff[0];
				if(lookup_spline3_alt( comdd_traj, 1, s->time + PREVIEW2_RHC_DELAY, NULL,&pData.x0[6], &temp, &temp ))
					//pData.x0[6] += s->comdd_d[1] - s->comdd_ff[1];

				//if(lookup_spline3_alt( com_traj, 0, s->time + PREVIEW2_RHC_DELAY, NULL,&temp, &temp, &pData.x0[2] ))
				//	pData.x0[2] += s->comdd_d[0] - s->comdd_ff[0];
				//if(lookup_spline3_alt( com_traj, 1, s->time + PREVIEW2_RHC_DELAY, NULL,&temp, &temp, &pData.x0[6] ))
				//	pData.x0[6] += s->comdd_d[1] - s->comdd_ff[1];

				break;

		}


#endif

		switch(s->controller_state){
			case STATE_SHIFTLEFT:
			case STATE_LIFTLEFT:
			case STATE_TOUCHLEFT:
			case STATE_STEPLEFT:
			case STATE_DOUBLESUPP:
				  pData.x0[3] = s->foot[LEFT][0];
				  pData.x0[7] = s->foot[LEFT][1];
				  pData.x0[8] = s->foot[RIGHT][0];
				  pData.x0[9] = s->foot[RIGHT][1];
				  break;
			case STATE_SHIFTRIGHT:
			case STATE_LIFTRIGHT:
			case STATE_TOUCHRIGHT:
			case STATE_STEPRIGHT:
				  pData.x0[3] = s->foot[RIGHT][0];
				  pData.x0[7] = s->foot[RIGHT][1];
				  pData.x0[8] = s->foot[LEFT][0];
				  pData.x0[9] = s->foot[LEFT][1];
				  break;
		}

		switch(s->controller_state){
			case STATE_SHIFTLEFT:
			case STATE_SHIFTRIGHT:
			case STATE_DOUBLESUPP:
				pData.t = s->cstate_elapsed_time;			
				break;
			case STATE_STEPLEFT:
			case STATE_STEPRIGHT:
				pData.t = s->cstate_elapsed_time + STEP_DSP_T;
				break;
			case STATE_LIFTLEFT:
			case STATE_LIFTRIGHT:
				pData.t = s->cstate_elapsed_time + STEP_DSP_T - LIFT_DELAY;
				break;
		}


		pData.start_time = s->time;
		pData.type = TYPE;

		//printf("% 7.3f: State Machine: Sending input data time = %f\n",SERVER_TIME,pData.start_time);

		preview_send(&pData);
	
	}
}



void transition_to( PRIMUS_t *s, int cstate, char *message )
{
	char buf[100];


//  if ( s->controller_print )
//    {
		sprintf(buf,"%s time = %f\n",message,s->time);
      printf( buf );
//    }

  s->cstate_start_time = s->time;
  s->cstate_elapsed_time = 0.0;
  s->controller_state = cstate;

  switch(cstate){
	case STATE_SHIFTLEFT:
	case STATE_SHIFTRIGHT:	
	case STATE_STEPLEFT:
	case STATE_STEPRIGHT:
	case STATE_LIFTLEFT:
	case STATE_LIFTRIGHT:
		run_preview(s,PREVIEW2_RHC);
		break;
  }

}

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
void init_state_machine(PRIMUS_t *s){
	static int initialized = 0;
	if (!initialized){
		//STARTUPINFO si;
		//PROCESS_INFORMATION pi;

		//ZeroMemory( &si, sizeof(si) );
		//si.cb = sizeof(si);
		//ZeroMemory( &pi, sizeof(pi) );

		//if( !CreateProcess( "..\\dbsimtcl\\preview.exe",   // No module name (use command line)
		//		NULL,			// Command line
		//		NULL,           // Process handle not inheritable
		//		NULL,           // Thread handle not inheritable
		//		FALSE,          // Set handle inheritance to FALSE
		//		0,              // No creation flags
		//		NULL,           // Use parent's environment block
		//		NULL,           // Use parent's starting directory 
		//		&si,            // Pointer to STARTUPINFO structure
		//		&pi )           // Pointer to PROCESS_INFORMATION structure
		//	) 
		//	{
		//		printf( "CreateProcess failed (%d).\n", GetLastError() );
		//		exit(-1);
		//	}

		preview_init();

		server_start_time = get_time();

		Sleep(1000);

		initialized = 1;
	}

	s->com_d[0] = s->com[0];
	s->com_d[1] = s->com[1];
	sdvset(0,0,0,s->comd_d);
	sdvset(0,0,0,s->comdd_d);
	sdvset(0,0,0,s->comdd_ff);
}

void
run_state_machine( PRIMUS_t *s )
{

	static double x0[10];
	static int step_count=0;
	int i;
	static int firsttime=1;

	if (s->time==0.0){
		clear_all_knots(foot_traj,3);
		init_trajectory(foot_traj,3);
		clear_all_knots(com_traj,2);
		init_trajectory(com_traj,2);
		clear_all_knots(comdd_traj,2);
		init_trajectory(comdd_traj,2);
		if(!firsttime)	preview_reinit();
		step_count = 0;
		sdvset(0,0,0,s->comdd_d);
		init_state_machine(s);
	}

	if (firsttime){
		firsttime=0;
	}


  s->cstate_elapsed_time = s->time - s->cstate_start_time;
  // printf( "%g %g %g\n", s->time, s->cstate_elapsed_time, s->cstate_start_time );

  /* Do actions and transitions */
  switch( s->controller_state )
    {
  //  case STATE_LAUNCH:
		//s->contact_state=CONTACT_DSP;
		//enable_ik = 0;

		//if (s->cstate_elapsed_time<2.0){
		//	s->des_state[6+L_HFE] += 0.1/2.0*0.001;
		//	s->des_state[6+L_KFE] += 0.2/2.0*0.001;
		//	s->des_state[6+L_AFE] += 0.1/2.0*0.001;

		//	s->des_state[6+R_HFE] += 0.1/2.0*0.001;
		//	s->des_state[6+R_KFE] += 0.2/2.0*0.001;
		//	s->des_state[6+R_AFE] += 0.1/2.0*0.001;

		//}

		//if (s->cstate_elapsed_time>=4.999)	{
		//	//transition_to(s,SHIFTRIGHT,"LAUNCH->SHIFTRIGHT\n");

		//	s->com_d[0] = s->com[0];
		//	s->com_d[1] = s->com[1];
		//	s->com_d[2] = s->com[2];

		//	s->foot_d[LEFT][0] = s->foot[LEFT][0];
		//	s->foot_d[LEFT][1] = s->foot[LEFT][1];
		//	s->foot_d[LEFT][2] = s->foot[LEFT][2];

		//	s->foot_d[RIGHT][0] = s->foot[RIGHT][0];
		//	s->foot_d[RIGHT][1] = s->foot[RIGHT][1];
		//	s->foot_d[RIGHT][2] = s->foot[RIGHT][2];

		//	s->des_state[0] = s->sdfast_state[0];
		//	s->des_state[1] = s->sdfast_state[1];
		//	s->des_state[2] = s->sdfast_state[2];
		//	s->des_state[3] = s->sdfast_state[3];
		//	s->des_state[4] = s->sdfast_state[4];
		//	s->des_state[5] = s->sdfast_state[5];
		//	for(i=0;i<N_JOINTS;i++)	s->des_state[6+i] = s->joints_true[i];
		//	s->des_state[N_U] = s->sdfast_state[N_U-4];

		//	set_state(m2,s->des_state);

		//	transition_to(s,STATE_HOME,"LAUNCH->HOME");
		//}
		//break;

	case STATE_HOME:
		s->contact_state=CONTACT_DSP;
		//enable_ik = 1;

		s->com_d[2] = 0.96;
		s->comd_d[2] = 0.0;

		sdvset(0,0,0,s->comdd_ff);
		sdvset(0,0,0,s->comd_d);

		s->com_d[0] = 0.5*(s->foot[LEFT][0] + s->foot[RIGHT][0]);
		s->com_d[1] = 0.5*(s->foot[LEFT][1] + s->foot[RIGHT][1]);

		if (s->time>=0.0){
			step_count = 0;
			transition_to(s,STATE_DOUBLESUPP,"HOME->DOUBLESUPP");
		}

		break;

	case STATE_SHIFTRIGHT:
		s->contact_state=CONTACT_DSP;

		run_preview(s,PREVIEW2_RHC);

		if (s->cstate_elapsed_time>=STEP_DSP_T-PREVIEW2_RHC_DELAY){			
//			if (s->foot_force[LEFT][2]<50){
				transition_to(s,STATE_STEPRIGHT,"SHIFTRIGHT->STEPRIGHT");
			//}else{
			//	transition_to(s,STATE_LIFTRIGHT,"SHIFTRIGHT->LIFTRIGHT");
			//}
		}
		break;

	case STATE_STEPRIGHT:

		s->contact_state=CONTACT_RSP;

		run_preview(s,PREVIEW2_RHC);

		if(s->cstate_elapsed_time>=0.5*STEP_TIME-PREVIEW2_RHC_DELAY && s->foot_force[LEFT][ZZ]>50){
			step_count++;
			transition_to(s,STATE_DOUBLESUPP,"STEPRIGHT->DOUBLESUPP");			
		}else if(s->cstate_elapsed_time>=STEP_TIME-0.001){
			transition_to(s,STATE_TOUCHRIGHT,"STEPRIGHT->TOUCHRIGHT");
		}
			
		break;

	case STATE_SHIFTLEFT:
		s->contact_state=CONTACT_DSP;
		
		run_preview(s,PREVIEW2_RHC);

		if (s->cstate_elapsed_time>=STEP_DSP_T-PREVIEW2_RHC_DELAY){			
			//if (s->foot_force[RIGHT][2]<50){
				transition_to(s,STATE_STEPLEFT,"SHIFTLEFT->STEPLEFT");
			//}else{
			//	transition_to(s,STATE_LIFTLEFT,"SHIFTLEFT->LIFTLEFT");
			//}
		}
		break;

	case STATE_STEPLEFT:

		s->contact_state=CONTACT_LSP;

		run_preview(s,PREVIEW2_RHC);

		if(s->cstate_elapsed_time>=0.5*STEP_TIME-PREVIEW2_RHC_DELAY && s->foot_force[RIGHT][ZZ]>50){
			step_count++;
			transition_to(s,STATE_DOUBLESUPP,"STEPLEFT->DOUBLESUPP");
		}else if(s->cstate_elapsed_time>=STEP_TIME-0.001){
			transition_to(s,STATE_TOUCHLEFT,"STEPLEFT->TOUCHLEFT");
		}
			
		break;

	case STATE_TOUCHLEFT:
		s->contact_state=CONTACT_DSP;

		if (s->cstate_elapsed_time>TOUCH_DELAY || s->foot_force[RIGHT][2]>50){			
			step_count++;
			transition_to(s,STATE_DOUBLESUPP,"TOUCHLEFT->DOUBLESUPP");
		}

		break;

	case STATE_TOUCHRIGHT:
		s->contact_state=CONTACT_DSP;

		if (s->cstate_elapsed_time>TOUCH_DELAY || s->foot_force[LEFT][2]>50){			
			step_count++;
			transition_to(s,STATE_DOUBLESUPP,"TOUCHRIGHT->DOUBLESUPP");
		}

		break;

	case STATE_LIFTLEFT:
		s->contact_state=CONTACT_DSP;

		run_preview(s,PREVIEW2_RHC);

		if (s->cstate_elapsed_time>=LIFT_DELAY || s->foot_force[RIGHT][2]<50){		
			transition_to(s,STATE_STEPLEFT,"LIFTLEFT->STEPLEFT");
		}

		break;

	case STATE_LIFTRIGHT:
		s->contact_state=CONTACT_DSP;

		run_preview(s,PREVIEW2_RHC);

		if (s->cstate_elapsed_time>=LIFT_DELAY || s->foot_force[LEFT][2]<50){		
			transition_to(s,STATE_STEPRIGHT,"LIFTRIGHT->STEPRIGHT");
		}

		break;

	case STATE_DOUBLESUPP:
			s->contact_state=CONTACT_DSP;
			//s->com_d[0] += 0.001*0.1*(0.5*(s->foot[LEFT][0] + s->foot[RIGHT][0]) - s->com_d[0]);
			//s->com_d[1] += 0.001*0.1*(0.5*(s->foot[LEFT][1] + s->foot[RIGHT][1]) - s->com_d[1]);
			
			sdvcopy(s->foot[LEFT],s->foot_d[LEFT]);
			sdvcopy(s->foot[RIGHT],s->foot_d[RIGHT]);


#ifdef STOP_AFTER_RECOVERED
			if (fabs(s->com[0] - 0.5*(s->foot[LEFT][0]+s->foot[RIGHT][0]) + s->comd[0]*sqrt(s->com[2]/9.81))<0.05 && fabs(s->com[1] - 0.5*(s->foot[LEFT][1]+s->foot[RIGHT][1]) + s->comd[1]*sqrt(s->com[2]/9.81))<0.1){
				run_preview(s,PREVIEW0_RHC);
				break;
			}	
#else
		if(s->time<1.0){
			run_preview(s,PREVIEW0_RHC);
			break;
		}	
#endif

		//break;
//
		//if (step_count>1){
		//	run_preview(s,PREVIEW0_RHC);
		//	break;
		//}


			//if (s->time<1.0 || s->cstate_elapsed_time<0.00) break;

			if (step_count%2==1){
				// step with right foot next
			  transition_to(s,STATE_SHIFTLEFT,"DOUBLESUPP->SHIFTLEFT");
			  
			}
			else{

			  transition_to(s,STATE_SHIFTRIGHT,"DOUBLESUPP->SHIFTRIGHT");

			}

		break;


    default:
      fprintf( stderr, "Unknown state %d in run_state_machine.\n",
	       s->controller_state );
    }


	if (s->time>0.00 && s->controller_state!=STATE_HOME){
		double temp;

		lookup_spline3_alt( com_traj, 0, s->time, NULL,
			&s->com_d[0], &s->comd_d[0], &temp );
		lookup_spline3_alt( com_traj, 1, s->time, NULL,
			&s->com_d[1], &s->comd_d[1], &temp );

		lookup_spline3_alt( comdd_traj, 0, s->time, NULL,
			&s->comdd_ff[0], &temp, &temp );
		lookup_spline3_alt( comdd_traj, 1, s->time, NULL,
			&s->comdd_ff[1], &temp, &temp );

		//sdvcopy(s->comdd_d,s->comdd_ff);
	}
  
	s->cop_d[0] = s->com_d[0] - 1.0/9.81*s->comdd_ff[0];
	s->cop_d[1] = s->com_d[1] - 1.0/9.81*s->comdd_ff[1];


	switch(s->controller_state){
		//case LIFTRIGHT:
		case STATE_STEPRIGHT:
			lookup_spline3_alt( foot_traj, 0, s->cstate_elapsed_time, NULL,
				&s->foot_d[LEFT][0], &s->footd_d[LEFT][0], &s->footdd_d[LEFT][0] );

			lookup_spline3_alt( foot_traj, 1, s->cstate_elapsed_time, NULL,
				&s->foot_d[LEFT][1], &s->footd_d[LEFT][1], &s->footdd_d[LEFT][1] );

			lookup_spline3_alt( foot_traj, 2, s->cstate_elapsed_time, NULL,
				&s->foot_d[LEFT][2], &s->footd_d[LEFT][2], &s->footdd_d[LEFT][2] );

			for (i=0;i<3;i++){
			  s->footd_d[RIGHT][i] = 0.0;
			  s->footdd_d[RIGHT][i] = 0.0;
			}

			break;

		//case LIFTLEFT:
		case STATE_STEPLEFT:
			lookup_spline3_alt( foot_traj, 0, s->cstate_elapsed_time, NULL,
				&s->foot_d[RIGHT][0], &s->footd_d[RIGHT][0], &s->footdd_d[RIGHT][0] );

			lookup_spline3_alt( foot_traj, 1, s->cstate_elapsed_time, NULL,
				&s->foot_d[RIGHT][1], &s->footd_d[RIGHT][1], &s->footdd_d[RIGHT][1] );

			lookup_spline3_alt( foot_traj, 2, s->cstate_elapsed_time, NULL,
				&s->foot_d[RIGHT][2], &s->footd_d[RIGHT][2], &s->footdd_d[RIGHT][2] );

			for (i=0;i<3;i++){
			  s->footd_d[LEFT][i] = 0.0;
			  s->footdd_d[LEFT][i] = 0.0;
			}

			break;

		default:
			for (i=0;i<3;i++){
			  s->footd_d[RIGHT][i] = 0.0;
			  s->footdd_d[RIGHT][i] = 0.0;
			  s->footd_d[LEFT][i] = 0.0;
			  s->footdd_d[LEFT][i] = 0.0;
			}
			s->foot_d[LEFT][2] = 0.0;
			s->foot_d[RIGHT][2] = 0.0;
			break;
	}


}

}